module cordic_apb
#(parameter integer unsigned WIDTH = 32,
  parameter integer unsigned DEPTH = 10,
  parameter integer unsigned FOLDED = 1)
(
  // Group: APB interface, clock and reset
  // Inputs:
  // Outputs: 
  input logic                 CLK,              // System Clock
  input logic                 RSTn,             // System Reset
  input logic [31:0]          PADDR,            // APB Address bus
  input logic                 PENABLE,          // APB bus enable
  input logic                 PSEL,             // APB slave select
  input logic [31:0]          PWDATA,           // APB write databus
  input logic                 PWRITE,           // APB write transfer
  output logic [31:0]         PRDATA,           // APB read databus 
  output logic                PREADY,           // APB ready (always 1)
  output logic                PSLVERR           // APB error (always 0)
);

if(FOLDED == 1)
begin
cordic_apb_folded #(WIDTH,DEPTH)
cordic_apb_folded (.CLK(CLK),
                   .RSTn(RSTn),
                   .PADDR(PADDR),
                   .PENABLE(PENABLE),
                   .PSEL(PSEL),
                   .PWDATA(PWDATA),
                   .PWRITE(PWRITE),
                   .PRDATA(PRDATA),
                   .PREADY(PREADY),
                   .PSLVERR(PSLVERR)
);
end
else
begin
cordic_apb_unfolded #(WIDTH,DEPTH)
cordic_apb_unfolded (.CLK(CLK),
                     .RSTn(RSTn),
                     .PADDR(PADDR),
                     .PENABLE(PENABLE),
                     .PSEL(PSEL),
                     .PWDATA(PWDATA),
                     .PWRITE(PWRITE),
                     .PRDATA(PRDATA),
                     .PREADY(PREADY),
                     .PSLVERR(PSLVERR)
);
end

endmodule
