module cordic_apb_folded
#(parameter integer unsigned WIDTH = 32,
  parameter integer unsigned DEPTH = 10)
(
  // Group: APB interface, clock and reset
  // Inputs:
  // Outputs: 
  input logic                 CLK,              // System Clock
  input logic                 RSTn,             // System Reset
  input logic [31:0]          PADDR,            // APB Address bus
  input logic                 PENABLE,          // APB bus enable
  input logic                 PSEL,             // APB slave select
  input logic [31:0]          PWDATA,           // APB write databus
  input logic                 PWRITE,           // APB write transfer
  output logic [31:0]         PRDATA,           // APB read databus 
  output logic                PREADY,           // APB ready (always 1)
  output logic                PSLVERR           // APB error (always 0)
);

`ifdef CORDIC_16
logic [15:0] x_data; 
logic [15:0] y_data; 

logic [15:0] cos_data;
logic [15:0] sin_data;
`elsif CORDIC_8
logic [7:0]  x_data; 
logic [7:0]  y_data; 

logic [7:0]  cos_data;
logic [7:0]  sin_data;
`else
logic [31:0] x_data; 
logic [31:0] y_data;

logic [31:0] cos_data;
logic [31:0] sin_data;
`endif

logic [31:0] z_data;
//logic en_from_cordic;

logic op;

logic step, conv, en_reg;
integer counter;

cordic_top_folded #(WIDTH, DEPTH)
cordic_top_folded (.clk(CLK),
                   //.en(en_to_cordic),
                   .op(op),
                   .rst(RSTn),
                   //.en_o(en_from_cordic),
                   .X_i(x_data),
                   .Y_i(y_data),
                   .Z_i(z_data[31:32-WIDTH]),
                   .X_o(cos_data),
                   .Y_o(sin_data),
                   .Z_o(),
                   .en_reg(en_reg),
                   .counter(counter),
                   .step(step),
                   .conv(conv));


cordic_apb_wrapper_folded #(DEPTH)
cordic_apb_wrapper_folded (.CLK(CLK),
                           .RSTn(RSTn),
                           .PADDR(PADDR),
                           .PENABLE(PENABLE),
                           .PSEL(PSEL),
                           .PWDATA(PWDATA),
                           .PWRITE(PWRITE),
                           .PRDATA(PRDATA),
                           .PREADY(PREADY),
                           .PSLVERR(PSLVERR),
                           //.en_i(en_from_cordic),
                           .X_o(x_data),
                           .Y_o(y_data),
                           .data_in_cos(cos_data),
                           .data_in_sin(sin_data),
                           .Z_o(z_data),
                           //.en_o(en_to_cordic),
                           .operation_o(op),
                           .en_reg(en_reg),
                           .counter(counter),
                           .step(step),
                           .conv(conv));

endmodule
