module cordic_top_folded
#(
  parameter integer unsigned WIDTH = 32,
  parameter integer unsigned DEPTH = 10
)
(
  // Group: clk and enable inputs
  // Inputs:
  //    clk - clock signal
  //    rst - reset signal
  input logic                clk,  
  //input logic                en,
  input logic                op,
  input logic                rst,
  //output logic               en_o,
  // Group: X, Y and Z inputs
  // Inputs:
  //    X - x position used for cosine
  //    Y - y position used for sine
  //    Z - used to determine the next rotation
  `ifdef CORDIC_16
  input shortint             X_i,  
  input shortint             Y_i,
  `elsif CORDIC_8
  input byte                 X_i,  
  input byte                 Y_i,
  `else
  input int                  X_i,  
  input int                  Y_i,
  `endif
  input logic [WIDTH-1:0]    Z_i,
  // Group: X, Y and Z outputs
  // Outputs:
  //    X - x position to post processing
  //    Y - y position to post processing
  //    Z - z to post processing
  `ifdef CORDIC_16
  output shortint            X_o,  
  output shortint            Y_o, 
  `elsif CORDIC_8
  output byte                X_o,  
  output byte                Y_o, 
  `else
  output int                 X_o,  
  output int                 Y_o,
  `endif
  output logic [WIDTH-1:0]   Z_o,
  input logic                en_reg,
  input integer              counter,
  input logic                step,
  input logic                conv   
);

//support signals
`ifdef CORDIC_16
shortint X_support;
shortint Y_support;
`elsif CORDIC_8
byte X_support;
byte Y_support;
`else
int X_support;
int Y_support;
`endif

logic [WIDTH-1:0] Z_support;

//final results
`ifdef CORDIC_16
shortint X_final;
shortint Y_final;
shortint X_stage;
shortint Y_stage;
`elsif CORDIC_8
byte X_final;
byte Y_final;
byte X_stage;
byte Y_stage;
`else
int X_final;
int Y_final;
int X_stage;
int Y_stage;
`endif

logic [WIDTH-1:0] Z_final;
logic [WIDTH-1:0] Z_stage;

logic neg_support;

cordic_pre #(WIDTH)
cordic_pre (.Z_i(Z_i),
            .Z_o(Z_support[WIDTH-1:0]),
            //.Z_o(Z_support [0][31:32-WIDTH]),
            .neg(neg_support)
);

cordic_stage_folded #(WIDTH,DEPTH)
cordic_stage_folded (.clk(clk),
                     //.en(en),
                     .op(op),
                     .rst(rst),
                     //.en_o(en_o),
                     .X_i(X_i),
                     .Y_i(Y_i),
                     .Z_i(Z_support[WIDTH-1:0]),
                     .X_o(X_stage),
                     .Y_o(Y_stage),
                     .Z_o(Z_stage),
                     .en_reg(en_reg),
                     .counter(counter),
                     .step(step),
                     .conv(conv) 
);

cordic_post #(WIDTH)
cordic_post (.en_i(1'b0),
             .X_i(X_stage),
             .Y_i(Y_stage),
             .Z_i(Z_stage),
             .neg_i(neg_support),
             .operation_i(op),
             .X_o(X_final),
             .Y_o(Y_final),
             .Z_o(Z_final),
             .en_o()
);

assign X_o = X_final;
assign Y_o = Y_final;
assign Z_o = Z_final;

endmodule

