module cordic_post 
#(
  parameter integer unsigned WIDTH = 32,
  parameter integer unsigned DEPTH = 10
)
(
  input                      en_i,
  // Group: X, Y and Z inputs
  // Inputs:
  //    X - x position used for cosine
  //    Y - y position used for sine
  //    Z - 
  `ifdef CORDIC_16
  input shortint             X_i,  
  input shortint             Y_i,
  `elsif CORDIC_8
  input byte                 X_i,  
  input byte                 Y_i,
  `else
  input int                  X_i,  
  input int                  Y_i,
  `endif
  input logic [WIDTH-1:0]    Z_i,
  input logic                neg_i,
  input logic                operation_i,
  // Group: X, Y and Z outputs
  // Outputs:
  //    X - cos
  //    Y - sin
  //    Z - 
  `ifdef CORDIC_16
  output shortint            X_o,  
  output shortint            Y_o, 
  `elsif CORDIC_8
  output byte                X_o,  
  output byte                Y_o, 
  `else
  output int                 X_o,  
  output int                 Y_o,
  `endif
  output logic [WIDTH-1:0]   Z_o, 
  output logic               en_o
);

assign en_o = en_i;

`ifdef CORDIC_16
shortint X_input, Y_input;
shortint X_pos, Y_pos, X_neg, Y_neg;
`elsif CORDIC_8
byte X_input, Y_input;
byte X_pos, Y_pos, X_neg, Y_neg; 
`else
int X_input, Y_input;
int X_pos, Y_pos, X_neg, Y_neg;
`endif

assign X_input = X_i;
assign Y_input = Y_i;

//scaling no longer needed, different starting point on fw
//assign X_pos = (X_input >>> 1) + (X_input >>> 3) - (X_input >>> 6) - (X_input >>> 9);
//assign Y_pos = (Y_input >>> 1) + (Y_input >>> 3) - (Y_input >>> 6) - (Y_input >>> 9);

assign X_pos = X_input;
assign Y_pos = Y_input;

assign X_neg = -X_pos;
assign Y_neg = -Y_pos;

assign X_o = operation_i ? X_i : (neg_i ? X_neg : X_pos);
assign Y_o = operation_i ? Y_i : (neg_i ? Y_neg : Y_pos);
assign Z_o = Z_i;

endmodule
