`timescale 1 ns/10 ps  // time-unit = 1 ns, precision = 10 ps

module rf_apb #(reg_file_depth = 2,
                reg_depth = 8)
              (
	      //standard APB interface (modport slave)
              interface                        APB,
              //CLK and RST signals
              input logic                      CLK,
              input logic                      RSTn
);

/////////////////////////internal signals&regs///////////////////////////////////////
logic [reg_depth-1:0] regs [2**reg_file_depth-1:0];

/////////////////////////functions//////////////////////////////////////////////////
//read access
function is_read();
   return APB.psel & APB.penable & ~APB.pwrite;
endfunction

//write access
function is_write();
   return APB.psel & APB.penable & APB.pwrite;
endfunction

/////////////////////////signal assignment//////////////////////////////////////////
assign APB.pready = 1;
assign APB.pslverr = 0; 

always_comb 
   if(is_read())
      APB.prdata = regs[APB.paddr[reg_file_depth+1:2]];
   

always_ff @(posedge CLK or negedge RSTn)
   if(!RSTn)
      regs = '{default:'0};
   else begin
      if(is_write())
         regs[APB.paddr[reg_file_depth+1:2]] = APB.pwdata;
   end 


endmodule
