`timescale 1ns/100ps
//fully synchronous rom model
//numByte is the number of Bytes of the memory (1 MiB  = 1024*1024 Bytes according to the specs in riscv_1_gp_01)
module rom #(parameter numByte = 512*1024)
            (input logic rom0_clk,
             input logic rom0_ce,
             input logic[($clog2(numByte)-2)-1:0] rom0_addr,
             output logic[31:0] rom0_rdata);

//1 MiB = 4 * 256 KiB => each memory location is made of 32 bits (1 word) instead of 1 byte
//(* rom_style = "block" *) logic [31:0] rom[(1<<(($clog2(numByte))-2))-1:0];
logic [31:0] rom[(1<<(($clog2(numByte))-2))-1:0];

//rom initialization
`ifdef TESTW
initial $readmemb("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/testw.mem", rom);
`elsif NOP
initial $readmemb("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/nop.mem", rom);
`elsif SEGGER
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/segger.mem", rom);
`elsif SEGGERAHB
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/seggerAHB.mem", rom);
`elsif ECLIPSE_VECT_100
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/eclipsevector.mem", rom);
`elsif ECLIPSE
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/eclipse.mem", rom);
`elsif ECLIPSE_MONTY
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/monty.mem", rom);
`elsif RF
initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/rf.mem", rom);
`endif
//initial $readmemh("/design/mproj/ih_sandbox/users/fpozzana/ih_sandbox.fpth_mxd/ih_sandbox.fpth/digital/fw/testmemh.mem", rom);

//read capability
//read the complete word if rom0_ce is high
always_ff @(posedge rom0_clk) begin
  if (rom0_ce)
    rom0_rdata <= rom[rom0_addr];
end

endmodule
