`timescale 1ns/100ps
//fully synchronous ram model
//numByte is the number of Bytes of the memory (128 KiB  = 128*1024 Bytes according to the specs in riscv_1_gp_01)
module rom #(parameter numByte = 128*1024)
            (input logic rom0_clk,
             input logic rom0_ce,
             input logic rom0_we,
             input logic[3:0] rom0_be,
             input logic[($clog2(numByte)-2)-1:0] rom0_addr,
             input logic[31:0] rom0_wdata,
             output logic[31:0] rom0_rdata);
             
//parameter INIT_FILE = "";
//128 KiB = 4 * 32 KiB => each memory location is made of 32 bits (1 word) instead of 1 byte
//(* ram_style = "block" *) logic[31:0] ram[(1<<(($clog2(numByte))-2))-1:0];
logic[31:0] rom[(1<<(($clog2(numByte))-2))-1:0];

//parameter rom_INIT_FILE = "";

//initial begin
//$readmemh(rom_INIT_FILE, rom);// 0, (1<<(($clog2(numByte))-2))-1);
//end

initial begin
//sanity tests
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/toggle_sanity_test.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/march_ram_test.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/toggle_sanity_test_adv.mem", rom);
//uart tests
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/uart_playback.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/uart_irq.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/uart_scanf.mem", rom);
//spi tests
$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/spi_send_byte.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/spi_send_byte_slow.mem", rom);
//$readmemh("C:/Users/federico.pozzana/Desktop/rv_digtop/rv_digtop.ip_user_files/mem_init_files/spi_irq_slow.mem", rom);
end

//write capability
//a memory location is written if
//ram0_ce and ram0_we are high
//the corresponding bit in ram0_be is high
always_ff @(posedge rom0_clk) begin
  if (rom0_ce) begin
    if (rom0_we)
      if (rom0_be[0])
        rom[rom0_addr][7:0] <= rom0_wdata[7:0];
      if (rom0_be[1])
        rom[rom0_addr][15:8] <= rom0_wdata[15:8];
      if (rom0_be[2])
        rom[rom0_addr][23:16] <= rom0_wdata[23:16];
      if (rom0_be[3])
        rom[rom0_addr][31:24] <= rom0_wdata[31:24];
    end
end

//read capability
//read the complete word if ram0_ce is high
always_ff @(posedge rom0_clk) begin
  if (rom0_ce)
    rom0_rdata <= rom[rom0_addr];
end

endmodule