/*
 * interrupt_handlers.c
 *
 *  Created on: 28 mag 2020
 *      Author: Federico.Pozzana
 */

#include "interrupt_handlers.h"
#include "riscv_1_gp_01.h"
#include "core_rv32.h"

#include "i2c.h"

/////////////////////lfsr start//////////////////////////////////////////////
/////////////////////master write////////////////////////////////////////////

static HALF_WORD lfsr = 0xACE1;
static HALF_WORD mask = 0xB400;
static HALF_WORD lfsr_next = 0xE270;

static BYTE op;
static BYTE next_op;

/////////////////////////////////////////////////////////////////////////////

BYTE lfsr_8(BYTE lfsr, BYTE mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

HALF_WORD lfsr_16(HALF_WORD lfsr, HALF_WORD mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

/////////////////////////////////////////////////////////////////////////////

void enable_interrupt(void) {
	IER = 0xFFFFFFFF;
}

void disable_interrupt(void) {
	IER = 0x00000000;
}

void enable_event(void) {
	EER = 0xFFFFFFFF;
}

void disable_event(void) {
	EER = 0x00000000;
}

//ICP = ICP | something => clear interrupt, not global (interrupt clear pending)

void  __attribute__((interrupt("machine")))
PowerFail_Handler(void) {
   //Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000001;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
BusError_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000002;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_02_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000004;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_03_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000008;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_04_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000010;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C master firmware/////////////////////////////////////
/////////////////////master read/////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

	BYTE test0;
	BYTE test1;
	BYTE test2;
	BYTE test3;
	BYTE test4;
	BYTE test5;
	BYTE test6;
	BYTE test7;

	i2c_clear_irq_stop();

	switch(op)
	{
	case 0 : test0 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff)){
				 test_check = 0x00;
			}
			break;
	case 1 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1)){
				 test_check = 0x00;
			}
			break;
	case 2 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2)){
				 test_check = 0x00;
			}
			break;
	case 3 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3)){
				 test_check = 0x00;
			}
			break;
	case 4 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3) | test4 != ((lfsr & 0x0ff0) >> 4)){
				 test_check = 0x00;
			}
			break;
	case 5 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 test5 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3) | test4 != ((lfsr & 0x0ff0) >> 4) | test5 != ((lfsr & 0x1fe0) >> 5)){
				 test_check = 0x00;
			}
			break;
	case 6 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 test5 = i2c_read_data();
			 test6 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3) | test4 != ((lfsr & 0x0ff0) >> 4) | test5 != ((lfsr & 0x1fe0) >> 5) | test6 != ((lfsr & 0x3fc0) >> 6)){
				 test_check = 0x00;
			 }
			 break;
	case 7 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 test5 = i2c_read_data();
			 test6 = i2c_read_data();
			 test7 = i2c_read_data();
			 if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3) | test4 != ((lfsr & 0x0ff0) >> 4) | test5 != ((lfsr & 0x1fe0) >> 5) | test6 != ((lfsr & 0x3fc0) >> 6) | test7 != ((lfsr & 0x7f80) >> 7)){
				 test_check = 0x00;
			 }
			 break;
	default : test0 = i2c_read_data();
			  test1 = i2c_read_data();
			  test2 = i2c_read_data();
			  test3 = i2c_read_data();
			  if(test0 != (lfsr & 0x00ff) | test1 != ((lfsr & 0x01fe) >> 1) | test2 != ((lfsr & 0x03fc) >> 2) | test3 != ((lfsr & 0x07f8) >> 3)){
				  test_check = 0x00;
			  }
			  break;
	}


	switch(next_op)
	{
	case 0 : i2c_set_rx_bytes_count(0x01);
			 break;
	case 1 : i2c_set_rx_bytes_count(0x02);
			 break;
	case 2 : i2c_set_rx_bytes_count(0x03);
			 break;
	case 3 : i2c_set_rx_bytes_count(0x04);
			 break;
	case 4 : i2c_set_rx_bytes_count(0x05);
			 break;
	case 5 : i2c_set_rx_bytes_count(0x06);
			 break;
	case 6 : i2c_set_rx_bytes_count(0x07);
			 break;
	case 7 : i2c_set_rx_bytes_count(0x08);
		     break;
	default : i2c_set_rx_bytes_count(0x03);
			  break;
	}

	test_check ^= 0x04;
	test_check ^= 0x04;


/////////////////////////////////////////////////////////////////////////////
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_05_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000020;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C master firmware/////////////////////////////////////
/////////////////////master read/////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

	i2c_master_read(0x01, 0x02);
	lfsr = lfsr_16(lfsr,mask);
	lfsr_next = lfsr_16(lfsr_next,mask);
	op = (lfsr & 0x0007);
	next_op = (lfsr_next & 0x0007);

	if(lfsr == 0xACE1){
		test_check = 0x02;
	}

/////////////////////////////////////////////////////////////////////////////
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_06_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000040;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_07_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000080;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_08_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000100;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_09_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000200;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_10_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000400;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_11_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000800;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_12_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00001000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_13_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00002000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_14_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00004000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_15_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00008000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_16_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00010000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_17_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00020000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_18_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00040000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_19_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00080000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_20_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00100000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_21_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00200000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_22_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00400000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_23_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00800000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_24_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x01000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_25_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x02000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_26_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x04000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_27_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x08000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_28_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x10000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_29_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x20000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_30_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x40000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_31_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x80000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}
