/*******************************************************************************
 * Copyright (C) 2016 Maxim Integrated Products, Inc., All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated
 * Products, Inc. shall not be used except as stated in the Maxim Integrated
 * Products, Inc. Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products, Inc. retains all
 * ownership rights.
 *
 * $Date: 2020/04/22 $
 * $Revision: #2 $
 *
 ******************************************************************************/

#ifndef _RISCV_1_GP_01_H_
#define _RISCV_1_GP_01_H_

#ifndef TARGET_NUM
#define TARGET_NUM  riscv_1_gp_01
#endif

#define MXC_NUMCORES 1

//#include "riscv_1_gp_01_config.h"

#include <stdint.h>

#ifndef  FALSE
#define  FALSE      (0)
#endif

#ifndef  TRUE
#define  TRUE       (1)
#endif

/* COMPILER SPECIFIC DEFINES (IAR, ARMCC and GNUC) */
#if defined ( __GNUC__ )
#define __weak __attribute__((weak))

#elif defined ( __CC_ARM)

#define inline __inline
#pragma anon_unions

#endif

typedef enum {
/*    NonMaskableInt_IRQn    = -14,
    HardFault_IRQn         = -13,
    MemoryManagement_IRQn  = -12,
    BusFault_IRQn          = -11,
    UsageFault_IRQn        = -10,
    SVCall_IRQn            = -5,
    DebugMonitor_IRQn      = -4,
    PendSV_IRQn            = -2,
    SysTick_IRQn           = -1,
*/
    /* Device-specific interrupt sources (external to ARM core)                 */
    /*                      table entry number                                  */
    /*                      ||||                                                */
    /*                      ||||  table offset address                          */
    /*                      vvvv  vvvvvv                                        */
    RSV0_IRQn = 0,           /* 0x00  0x0000  00: Reserved */
    RSV1_IRQn,               /* 0x00  0x0000  01: Reserved */
    RSV2_IRQn,               /* 0x00  0x0000  02: Reserved */
    RSV3_IRQn,               /* 0x00  0x0000  03: Reserved */
    RSV4_IRQn,               /* 0x00  0x0000  04: Reserved */
    RSV5_IRQn,               /* 0x00  0x0000  05: Reserved */
    RSV6_IRQn,               /* 0x00  0x0000  06: Reserved */
    RSV7_IRQn,               /* 0x00  0x0000  07: Reserved */
    RSV8_IRQn,               /* 0x00  0x0000  08: Reserved */
    RSV9_IRQn,               /* 0x00  0x0000  09: Reserved */
    RSV10_IRQn,              /* 0x00  0x0000  10: Reserved */
    RSV11_IRQn,              /* 0x00  0x0000  11: Reserved */
    RSV12_IRQn,              /* 0x00  0x0000  12: Reserved */
    RSV13_IRQn,              /* 0x00  0x0000  13: Reserved */
    GPIOWAKE_IRQn,           /* 0x46  0x0118  14: GPIO Wakeup */
    TPU_IRQn,                /* 0x00  0x0000  15: Crypto */
    PUFC_IRQn,               /* 0x00  0x0000  16: PUFC */
    RSV17_IRQn,              /* 0x00  0x0000  17: Reserved */
    RSV18_IRQn,              /* 0x00  0x0000  18: Reserved */
    FLC_IRQn,                /* 0x27  0x009C  21: Flash Controller */
    SPI1_IRQn,               /* 0x20  0x0080  22: SPI-MSS */
    SPI0_IRQn,               /* 0x20  0x0080  23: SPI0 */
    RSV24_IRQn,              /* 0x00  0x0000  24: Reserved */
    I2C0_IRQn,               /* 0x1D  0x0074  25: I2C0 */
    UART1_IRQn,              /* 0x00  0x0000  26: UART1 */
    UART0_IRQn,              /* 0x00  0x0000  27: UART0 */
    TMR1_IRQn,               /* 0x00  0x0000  28: Timer 1 */
    TMR0_IRQn,               /* 0x15  0x0054  29: Timer 0 */
    GPIO1_IRQn,              /* 0x00  0x0000  19: GPIO1 */
    GPIO0_IRQn,              /* 0x28  0x00A0  20: GPIO0 */
    WDT0_IRQn,               /* 0x11  0x0044  30: Watchdog 0 */
    PF_IRQn = 0,             /* 0x10  0x0040  31: Power Fail */

    /* RISCV-specific interrupt sources (internal to core)                 */

    ResetHandler_IRQn,       /* 0x10  0x0040  33: Reset Handler */
    IllegalINSN_IRQn,        /* 0x10  0x0040  33: Illegal Instruction Handler */
    ECallINSN_IRQn,          /* 0x10  0x0040  33: Reset Handler */
    MXC_IRQ_EXT_COUNT,

//   PF_IRQn = 0,            /* 0x10  0x0040  16: Power Fail */
//   USB_IRQn,               /* 0x12  0x0048  18: USB */
//   RTC_IRQn,               /* 0x13  0x004C  19: RTC */
//   TRNG_IRQn,              /* 0x14  0x0050  20: True Random Number Generator */
//   TMR0_IRQn,              /* 0x15  0x0054  21: Timer 0 */
//   TMR1_IRQn,              /* 0x16  0x0058  22: Timer 1 */
//   TMR2_IRQn,              /* 0x17  0x005C  23: Timer 2 */
//   TMR3_IRQn,              /* 0x18  0x0060  24: Timer 3*/
//   TMR4_IRQn,              /* 0x19  0x0064  25: Timer 4*/
//   TMR5_IRQn,              /* 0x1A  0x0068  26: Timer 5 */
//   SC0_IRQn,               /* 0x1B  0x006C  27: Smart Card 0 */
//   LCD_IRQn,               /* 0x1C  0x0070  28: LCD Controller*/
//   I2C0_IRQn,              /* 0x1D  0x0074  29: I2C0 */
//   UART0_IRQn,             /* 0x1E  0x0078  30: UART 0 */
//   UART1_IRQn,             /* 0x1F  0x007C  31: UART 1 */
//   SPI0_IRQn,              /* 0x20  0x0080  32: SPI0 */
//   SPI1_IRQn,              /* 0x21  0x0084  33: SPI1 */
//   SPI2_IRQn,              /* 0x22  0x0088  34: SPI2 */
//   SKBD_IRQn,              /* 0x23  0x008C  35: Secure Keypad */
//   ADC_IRQn,               /* 0x24  0x0090  36: ADC */
//   RSV21_IRQn,             /* 0x25  0x0094  37: Reserved */
//   MSRADC_IRQn,            /* 0x26  0x0098  38: Magstripe DSP */
//   FLC0_IRQn,              /* 0x27  0x009C  39: Flash Controller 0 */
//   GPIO0_IRQn,             /* 0x28  0x00A0  40: GPIO0 */
//   GPIO1_IRQn,             /* 0x29  0x00A4  41: GPIO2 */
//   GPIO2_IRQn,             /* 0x2A  0x00A8  42: GPIO3 */
//   CRYPTO_IRQn,            /* 0x2B  0x00AC  43: Crypto */
//   DMA0_IRQn,              /* 0x2C  0x00B0  44: DMA0 */
//   DMA1_IRQn,              /* 0x2D  0x00B4  45: DMA1 */
//   DMA2_IRQn,              /* 0x2E  0x00B8  46: DMA2 */
//   DMA3_IRQn,              /* 0x2F  0x00BC  47: DMA3 */
//   TMR6_IRQn,              /* 0x30  0x00C0  48: Reserved */
//   TMR7_IRQn,              /* 0x31  0x00C4  49: Reserved */
//   UART2_IRQn,             /* 0x32  0x00C8  50: UART 2 */
//   NFC_IRQn,               /* 0x33  0x00CC  51: NFC Link Control */
//   I2C1_IRQn,              /* 0x34  0x00D0  52: I2C1 */
//   SC1_IRQn,               /* 0x35  0x00D4  53: Smart Card 1 */
//   SPIXC_IRQn,             /* 0x36  0x00D8  54: SPI execute in place */
//   RSV39_IRQn,             /* 0x37  0x00DC  55: Reserved */
//   RSV40_IRQn,             /* 0x38  0x00E0  56: Reserved */
//   RSV41_IRQn,             /* 0x39  0x00E4  57: Reserved */
//   RSV42_IRQn,             /* 0x3A  0x00E8  58: Reserved */
//   RSV43_IRQn,             /* 0x3B  0x00EC  59: Reserved */
//   RSV44_IRQn,             /* 0x3C  0x00F0  60: Reserved */
//   RSV45_IRQn,             /* 0x3D  0x00F4  61: Reserved */
//   RSV46_IRQn,             /* 0x3E  0x00F8  62: Reserved */
//   RSV47_IRQn,             /* 0x3F  0x00FC  63: Reserved */
//   RSV48_IRQn,             /* 0x40  0x0100  64: Reserved */
//   RSV49_IRQn,             /* 0x41  0x0104  65: Reserved */
//   RSV50_IRQn,             /* 0x42  0x0108  66: Reserved */
//   RSV51_IRQn,             /* 0x43  0x010C  67: Reserved */
//   RSV52_IRQn,             /* 0x44  0x0110  68: Reserved */
//   RSV53_IRQn,             /* 0x45  0x0114  69: Reserved */
//   GPIOWAKE_IRQn,          /* 0x46  0x0118  70: GPIO Wakeup */
//   NFCWAKE_IRQn,           /* 0x47  0x011C  71: NFC Wakeup */
//   SPI3_IRQn,              /* 0x48  0x0120  72: SPI3 */
//   WDT1_IRQn,              /* 0x49  0x0124  73: Watchdog 1 */
//   GPIO3_IRQn,             /* 0x4A  0x0128  74: GPIO3 */
//   PT_IRQn,                /* 0x4B  0x012C  75: Pulse train */
//   SDMA0_IRQn,             /* 0x4C  0x0130  76: Smart DMA 0 */
//   HPB_IRQn,               /* 0x4D  0x0134  77: HyperBus */
//   I2C2_IRQn,              /* 0x4E  0x0138  78: I2C2 */
//   SDMA1_IRQn,             /* 0x4F  0x013C  79: Smart DMA 1 */
//   ETH_IRQn,               /* 0x50  0x0140  80: Ethernet MAC */
//   PBM_IRQn,               /* 0x51  0x0144  81: PBM */
//   SDHC_IRQn,              /* 0x52  0x0148  82: SDIO/SDHC */
//   OWM_IRQn,               /* 0x53  0x014C  83: One Wire Master */
//   DMA4_IRQn,              /* 0x54  0x0150  84: DMA4 */
//   DMA5_IRQn,              /* 0x55  0x0154  85: DMA5 */
//   DMA6_IRQn,              /* 0x56  0x0158  86: DMA6 */
//   DMA7_IRQn,              /* 0x57  0x015C  87: DMA7 */
//   DMA8_IRQn,              /* 0x58  0x0160  88: DMA8 */
//   DMA9_IRQn,              /* 0x59  0x0164  89: DMA9 */
//   DMA10_IRQn,             /* 0x5A  0x0168  90: DMA10 */
//   DMA11_IRQn,             /* 0x5B  0x016C  91: DMA11 */
//   DMA12_IRQn,             /* 0x5C  0x0170  92: DMA12 */
//   DMA13_IRQn,             /* 0x5D  0x0174  93: DMA13 */
//   DMA14_IRQn,             /* 0x5E  0x0178  94: DMA14 */
//   DMA15_IRQn,             /* 0x5F  0x017C  95: DMA15 */
//   USBDMA_IRQn,            /* 0x60  0x0180  96: USB DMA */
//   RSV81_IRQn,             /* 0x61  0x0184  97: Reserved */
//   ECC_IRQn,               /* 0x62  0x0188  98: Error Correction */
//   DVS_IRQn,               /* 0x63  0x018C  99: DVS Controller */
//   SIMO_IRQn,              /* 0x64  0x0190 100: SIMO Controller */
//   SCA_IRQn,               /* 0x65  0x0194  101: SCA */
//   RSV86_IRQn,             /* 0x66  0x0198  102: Reserved */
//   FLC1_IRQn,              /* 0x67  0x019C  103: Flash Controller 1 */
//   UART3_IRQn,             /* 0x68  0x01A0  104: UART 3 */
//   UART4_IRQn,             /* 0x69  0x01A4  105: UART 4 */
//   UART5_IRQn,             /* 0x6A  0x01A8  106: UART 5 */
//   CameraIF_IRQn,          /* 0x6B  0x01AC  107: Camera IF */
//   RSV92_IRQn,             /* 0x6C  0x01B0  108: Reserved */
//   HTIMER0_IRQn,           /* 0x6D  0x01B4  109: HTIMER0 */
//   HTIMER1_IRQn,           /* 0x6E  0x01B8  110: HTIMER1 */
//   MXC_IRQ_EXT_COUNT,
} IRQn_Type;

//#define MXC_IRQ_COUNT (MXC_IRQ_EXT_COUNT + 16)
#define MXC_IRQ_COUNT (MXC_IRQ_EXT_COUNT)


/* ================================================================================ */
/* ================      Processor and Core Peripheral Section     ================ */
/* ================================================================================ */

/* ----------------------  Configuration of the Cortex-M Processor and Core Peripherals  ---------------------- */
#define __CM4_REV                       0x0100          /*!< Cortex-M4 Core Revision                                */
#define __MPU_PRESENT                   1               /*!< MPU present or not                                     */
#define __NVIC_PRIO_BITS                3               /*!< Number of Bits used for Priority Levels                */
#define __Vendor_SysTickConfig          0               /*!< Set to 1 if different SysTick Config is used           */
#define __FPU_PRESENT                   1               /*!< FPU present or not                                     */

//#include <core_rv32.h>                                  /*!< Cortex-M4 processor and core peripherals               */
#include "system.h"                                     /*!< System Header                                          */


/* ================================================================================ */
/* ==================       Device Specific Memory Section       ================== */
/* ================================================================================ */

#define MXC_ROM_MEM_BASE                0x00000000UL
/*#define MXC_ROM_MEM_SIZE                (96UL * 1024)*/
/* #define MXC_FLASH0_MEM_BASE             0x00000000UL */
/* #define MXC_FLASH_MEM_BASE              MXC_FLASH0_MEM_BASE */
/* #define MXC_FLASH_PAGE_SIZE             0x00001000UL */
/* #define MXC_FLASH_MEM_SIZE              0x00010000UL */
/* #define MXC_INFO0_MEM_BASE              0x10800000UL */
/* #define MXC_INFO_MEM_BASE               MXC_INFO0_MEM_BASE */
/* #define MXC_INFO_MEM_SIZE               0x00002000UL */
#define MXC_SRAM_MEM_BASE               0x20000000UL
/*#define MXC_SRAM_MEM_SIZE               (2UL * 1024)*/

/* ================================================================================ */
/* ================       Device Specific Peripheral Section       ================ */
/* ================================================================================ */

/*
   Base addresses and configuration settings for all MAX325XX peripheral modules.
*/

/******************************************************************************/
/*                                                             Global control */
//#define MXC_BASE_GCR                    ((uint32_t)0x40000000UL)
//#define MXC_GCR                         ((mxc_gcr_regs_t*)MXC_BASE_GCR)

/******************************************************************************/
/*                                            Non-battery backed SI Registers */
//#define MXC_BASE_SIR                    ((uint32_t)0x40000400UL)
//#define MXC_SIR                         ((mxc_sir_regs_t*)MXC_BASE_SIR)

/******************************************************************************/
/*                                                           Function Control */
//#define MXC_BASE_FCR                  ((uint32_t)0x40000800UL)
//#define MXC_FCR                       ((mxc_nbbfc_regs_t*)MXC_BASE_FCR)

/******************************************************************************/
/*                                                                     Crypto */
//#define MXC_BASE_TPU                    ((uint32_t)0x40001000UL)
//#define MXC_TPU                         ((mxc_tpu_regs_t*)MXC_BASE_TPU)

//#define MXC_BASE_CRYPTO                  MXC_BASE_TPU
//#define MXC_CRYPTO                      ((mxc_crypto_regs_t*)MXC_BASE_CRYPTO)

/******************************************************************************/
/*                                                                   Watchdog */
//#define MXC_BASE_WDT0                   ((uint32_t)0x40003000UL)
//#define MXC_WDT0                        ((mxc_wdt_regs_t*)MXC_BASE_WDT0)

/******************************************************************************/
/*                                                           Security Monitor */
//#define MXC_BASE_SMON                   ((uint32_t)0x40004000UL)
//#define MXC_SMON                        ((mxc_smon_regs_t*)MXC_BASE_SMON)


/******************************************************************************/
/*                                                           Security Monitor */
//#define MXC_BASE_SMON                   ((uint32_t)0x40004000UL)
//#define MXC_SMON                        ((mxc_smon_regs_t*)MXC_BASE_SMON)

/******************************************************************************/
/*                                                            MISC Control Regs */
//#define MXC_BASE_MCR                    ((uint32_t)0x40006C00UL)
//#define MXC_MCR                         ((mxc_mcr_regs_t*)MXC_BASE_MCR)

/******************************************************************************/
/*                                                            PUF */
//#define MXC_BASE_PUF                    ((uint32_t)0x40007000UL)
// #define MXC_PUF                         ((mxc_puf_regs_t*)MXC_BASE_PUF)

// /******************************************************************************/
// /*                                                                       GPIO */
// #define MXC_CFG_GPIO_INSTANCES          (1)
// #define MXC_CFG_GPIO_PINS_PORT          (16)

// #define MXC_BASE_GPIO0                  ((uint32_t)0x40008000UL)
// #define MXC_GPIO0                       ((mxc_gpio_regs_t*)MXC_BASE_GPIO0)

// #define MXC_GPIO_GET_IDX(p)             ((p) == MXC_GPIO0 ? 0 :    -1)

// #define MXC_GPIO_GET_GPIO(i)            ((i) == 0 ? MXC_GPIO0 :    0)

// #define MXC_GPIO_GET_IRQ(i)             ((i) == 0 ? GPIO0_IRQn : 0)


// /******************************************************************************/
// /*                                                                      Timer */
// #define MXC_CFG_TMR_INSTANCES           (1)

// #define MXC_BASE_TMR0                   ((uint32_t)0x40010000UL)
// #define MXC_TMR0                        ((mxc_tmr_regs_t*)MXC_BASE_TMR0)

// #define MXC_TMR_GET_IRQ(i)              (IRQn_Type)((i) == 0 ? TMR0_IRQn :  0)

// #define MXC_TMR_GET_BASE(i)             ((i) == 0 ? MXC_BASE_TMR0 :    0)

// #define MXC_TMR_GET_TMR(i)              ((i) == 0 ? MXC_TMR0 :     0)

// #define MXC_TMR_GET_IDX(p)              ((p) == MXC_TMR0 ? 0 :     -1)

// /******************************************************************************/
// /*                                                                        SPIMSS */

// #define MXC_SPIMSS_INSTANCES               (1)
// #define MXC_SPIMSS_SS_INSTANCES            (1)
// #define MXC_SPIMSS_FIFO_DEPTH              (16)

// #define MXC_BASE_SPIMSS0                   ((uint32_t)0x40018000UL)
// #define MXC_SPIMSS0                        ((mxc_spimss_regs_t*)MXC_BASE_SPIMSS0)


// #define MXC_SPIMSS_GET_IDX(p)            ((p) == MXC_SPIMSS0 ? 0 :  -1)

// #define MXC_SPIMSS_GET_BASE(i)           ((i) == 0 ? MXC_BASE_SPIMSS0 :  0)

// #define MXC_SPIMSS_GET_SPI17Y(i)         ((i) == 0 ? MXC_SPIMSS0 : 0)

// #define MXC_SPIMSS_GET_IRQ(i)           (IRQn_Type)((i) == 0 ? SPIMSS0_IRQn : 0)

// /******************************************************************************/
// /*                                                              UART Classic  */
// #define MXC_UARTC_INSTANCES             (1)
// #define MXC_UARTC_FIFO_DEPTH            (8)

// #define MXC_BASE_UARTC0                 ((uint32_t)0x40020000UL)
// #define MXC_UARTC0                      ((mxc_uartc_regs_t*)MXC_BASE_UARTC0)

// #define MXC_UARTC_GET_IRQ(i)            (IRQn_Type)((i) == 0 ? UART0_IRQn : 0)

// #define MXC_UARTC_GET_BASE(i)           ((i) == 0 ? MXC_BASE_UARTC0 : 0)

// #define MXC_UARTC_GET_UARTC(i)          ((i) == 0 ? MXC_UARTC0 : 0)

// #define MXC_UARTC_GET_IDX(p)            ((p) == MXC_UARTC0 ? 0 : -1)

// /******************************************************************************/
// /*                                                              LPUART */
// #define MXC_LPUART_INSTANCES            (1)
// #define MXC_LPUART_FIFO_DEPTH           (8)

// #define MXC_BASE_LPUART                 ((uint32_t)0x40021000UL)
// #define MXC_LPUART                      ((mxc_lpuart_regs_t*)MXC_BASE_LPUART)

// #define MXC_LPUART_GET_IRQ(i)           (IRQn_Type)((i) == 0 ? UART1_IRQn : 0)

// #define MXC_LPUART_GET_BASE(i)          ((i) == 0 ? MXC_BASE_LPUART : 0)

// #define MXC_LPUART_GET_LPUART(i)        ((i) == 0 ? MXC_LPUART : 0)

// #define MXC_LPUART_GET_IDX(p)           ((p) == MXC_LPUART ? 0 : -1)


// /******************************************************************************/
// /*                                                                        I2C */
// #define MXC_I2C_INSTANCES               (1)

// #define MXC_I2C_FIFO_DEPTH              (8)

// #define MXC_BASE_I2C0                   ((uint32_t)0x4001D000UL)
// #define MXC_I2C0                        ((mxc_i2c_regs_t*)MXC_BASE_I2C0)


// #define MXC_I2C_GET_IRQ(i)              (IRQn_Type)((i) == 0 ? I2C0_IRQn : 0)

// #define MXC_I2C_GET_BASE(i)             ((i) == 0 ? MXC_BASE_I2C0 : 0)

// #define MXC_I2C_GET_TMR(i)              ((i) == 0 ? MXC_I2C0 : 0)

// #define MXC_I2C_GET_IDX(p)              ((p) == MXC_I2C0 ? 0 : -1)

// #define MXC_I2C_GET_I2C(p)              ((p) == 0x0 ? MXC_I2C0 : 0)

// /******************************************************************************/
// /*                                                                        FLC */
// #define MXC_FLC_INSTANCES               (1)

// #define MXC_BASE_FLC0                    ((uint32_t)0x40029000UL)
// #define MXC_FLC0                         ((mxc_flc_regs_t*)MXC_BASE_FLC0)

// #define MXC_FLC_GET_IRQ(i)             (IRQn_Type)((i) == 0 ? FLC0_IRQn : 0)

// #define MXC_FLC_GET_BASE(i)            ((i) == 0 ? MXC_BASE_FLC0 : 0)

// #define MXC_FLC_GET_FLC(i)            ((i) == 0 ? MXC_FLC0 : 0)

// #define MXC_FLC_GET_IDX(p)             ((p) == MXC_FLC0 ? 0 : -1)

// /******************************************************************************/
// /*                                                                        SPI */

// #define MXC_SPI17Y_INSTANCES               (1)
// #define MXC_SPI17Y_SS_INSTANCES            (4)
// #define MXC_SPI17Y_FIFO_DEPTH              (32)

// #define MXC_BASE_SPI17Y0                   ((uint32_t)0x40046000UL)
// #define MXC_SPI17Y0                        ((mxc_spi17y_regs_t*)MXC_BASE_SPI17Y0)



// #define MXC_SPI17Y_GET_IDX(p)            ((p) == MXC_SPI17Y0 ? 0 : -1)

// #define MXC_SPI17Y_GET_BASE(i)           ((i) == 0 ? MXC_BASE_SPI17Y0 : 0)

// #define MXC_SPI17Y_GET_SPI17Y(i)         ((i) == 0 ? MXC_SPI17Y0 : 0)

// #define MXC_SPI17Y_GET_IRQ(i)           (IRQn_Type)((i) == 0 ? SPI17Y0_IRQn : 0)

/******************************************************************************/
/*                                                          APB Event Handler */ // HACK NI- add APB event handler here- but don't think this is right
//#define MXC_BASE_EVENT                  ((uint32_t)0xE0000000UL)
#define MXC_BASE_SDMA0                  ((uint32_t)0x40036000UL)
//#define MXC_BASE_EVENT                  MXC_BASE_SDMA0
//#define MXC_EVENT                       ((mxc_event_regs_t *)MXC_BASE_EVENT)

/*                                                               Bit Shifting */

#define MXC_F_BIT_0                     (1 << 0)
#define MXC_F_BIT_1                     (1 << 1)
#define MXC_F_BIT_2                     (1 << 2)
#define MXC_F_BIT_3                     (1 << 3)
#define MXC_F_BIT_4                     (1 << 4)
#define MXC_F_BIT_5                     (1 << 5)
#define MXC_F_BIT_6                     (1 << 6)
#define MXC_F_BIT_7                     (1 << 7)
#define MXC_F_BIT_8                     (1 << 8)
#define MXC_F_BIT_9                     (1 << 9)
#define MXC_F_BIT_10                    (1 << 10)
#define MXC_F_BIT_11                    (1 << 11)
#define MXC_F_BIT_12                    (1 << 12)
#define MXC_F_BIT_13                    (1 << 13)
#define MXC_F_BIT_14                    (1 << 14)
#define MXC_F_BIT_15                    (1 << 15)
#define MXC_F_BIT_16                    (1 << 16)
#define MXC_F_BIT_17                    (1 << 17)
#define MXC_F_BIT_18                    (1 << 18)
#define MXC_F_BIT_19                    (1 << 19)
#define MXC_F_BIT_20                    (1 << 20)
#define MXC_F_BIT_21                    (1 << 21)
#define MXC_F_BIT_22                    (1 << 22)
#define MXC_F_BIT_23                    (1 << 23)
#define MXC_F_BIT_24                    (1 << 24)
#define MXC_F_BIT_25                    (1 << 25)
#define MXC_F_BIT_26                    (1 << 26)
#define MXC_F_BIT_27                    (1 << 27)
#define MXC_F_BIT_28                    (1 << 28)
#define MXC_F_BIT_29                    (1 << 29)
#define MXC_F_BIT_30                    (1 << 30)
#define MXC_F_BIT_31                    (1 << 31)

// /******************************************************************************/
// /*                                                               Bit Banding  */


// #define BITBAND(reg, bit)               ((0xf0000000 & (uint32_t)(reg)) + 0x2000000 + \
//                                             (((uint32_t)(reg) & 0x0fffffff) << 5) + ((bit) << 2))

// #define MXC_CLRBIT(reg, bit)            (*(volatile uint32_t *)BITBAND(reg, bit) = 0)
// #define MXC_SETBIT(reg, bit)            (*(volatile uint32_t *)BITBAND(reg, bit) = 1)
// #define MXC_GETBIT(reg, bit)            (*(volatile uint32_t *)BITBAND(reg, bit))

// #define MXC_SETFIELD(reg, mask, setting)  (reg = (reg & ~mask) | (setting & mask))

// /******************************************************************************/
// /*                                                                  SCB CPACR */

// /* Note: Added by Maxim Integrated, as these are missing from CMSIS/Core/Include/core_cm4.h */
// #define SCB_CPACR_CP10_Pos              20                              /*!< SCB CPACR: Coprocessor 10 Position */
// #define SCB_CPACR_CP10_Msk              (0x3UL << SCB_CPACR_CP10_Pos)   /*!< SCB CPACR: Coprocessor 10 Mask */
// #define SCB_CPACR_CP11_Pos              22                              /*!< SCB CPACR: Coprocessor 11 Position */
// #define SCB_CPACR_CP11_Msk              (0x3UL << SCB_CPACR_CP11_Pos)   /*!< SCB CPACR: Coprocessor 11 Mask */

#endif  /* _MAX325XX_REGS_H_ */
