/*
 * i2c.h
 *
 *  Created on: 23 lug 2020
 *      Author: Federico.Pozzana
 */

#ifndef I2C_H_
#define I2C_H_

#include "pulpino.h"

#define I2C_CTRL                      ( I2C_BASE_ADDR + 0x0000 )
#define I2C_STAT                      ( I2C_BASE_ADDR + 0x0004 )
#define I2C_INT0                      ( I2C_BASE_ADDR + 0x0008 )
#define I2C_INTE0                     ( I2C_BASE_ADDR + 0x000C )
#define I2C_INT1                      ( I2C_BASE_ADDR + 0x0010 )
#define I2C_INTE1                     ( I2C_BASE_ADDR + 0x0014 )
#define I2C_FIFO_CFG                  ( I2C_BASE_ADDR + 0x0018 )
#define I2C_RX_CFG                    ( I2C_BASE_ADDR + 0x001C )
#define I2C_RX_REG                    ( I2C_BASE_ADDR + 0x0020 )
#define I2C_TX_CFG                    ( I2C_BASE_ADDR + 0x0024 )
#define I2C_TX_REG                    ( I2C_BASE_ADDR + 0x0028 )
#define I2C_DATA                      ( I2C_BASE_ADDR + 0x002C )
#define I2C_MASTER_CTRL               ( I2C_BASE_ADDR + 0x0030 )
#define I2C_CLK_LOW                   ( I2C_BASE_ADDR + 0x0034 )
#define I2C_CLK_HIGH                  ( I2C_BASE_ADDR + 0x0038 )
#define I2C_HS_MODE                   ( I2C_BASE_ADDR + 0x003C )
#define I2C_TIMEOUT                   ( I2C_BASE_ADDR + 0x0040 )
#define I2C_RES0                      ( I2C_BASE_ADDR + 0x0044 )
#define I2C_DMA                       ( I2C_BASE_ADDR + 0x0048 )
#define I2C_SLAVE0                    ( I2C_BASE_ADDR + 0x004C )
#define I2C_SLAVE1                    ( I2C_BASE_ADDR + 0x0050 )
#define I2C_SLAVE2                    ( I2C_BASE_ADDR + 0x0054 )
#define I2C_SLAVE3                    ( I2C_BASE_ADDR + 0x0058 )
#define I2C_SLAVE4                    ( I2C_BASE_ADDR + 0x005C )
#define I2C_SLAVE5                    ( I2C_BASE_ADDR + 0x0060 )

#define CTRL_I2C                      REG(I2C_CTRL)
#define STAT_I2C                      REG(I2C_STAT)
#define INT0_I2C                      REG(I2C_INT0)
#define INTE0_I2C                     REG(I2C_INTE0)
#define INT1_I2C                      REG(I2C_INT1)
#define INTE1_I2C                     REG(I2C_INTE1)
#define FIFO_CFG_I2C                  REG(I2C_FIFO_CFG)
#define RX_CFG_I2C                    REG(I2C_RX_CFG)
#define RX_REG_I2C                    REG(I2C_RX_REG)
#define TX_CFG_I2C                    REG(I2C_TX_CFG)
#define TX_REG_I2C                    REG(I2C_TX_REG)
#define DATA_I2C                      REG(I2C_DATA)
#define MASTER_CTRL_I2C               REG(I2C_MASTER_CTRL)
#define CLK_LOW_I2C                   REG(I2C_CLK_LOW)
#define CLK_HIGH_I2C                  REG(I2C_CLK_HIGH)
#define HS_MODE_I2C                   REG(I2C_HS_MODE)
#define TIMEOUT_I2C                   REG(I2C_TIMEOUT)
#define RES0_I2C                      REG(I2C_RES0)
#define DMA_I2C                       REG(I2C_DMA)
#define SLAVE0_I2C                    REG(I2C_SLAVE0)
#define SLAVE1_I2C                    REG(I2C_SLAVE1)
#define SLAVE2_I2C                    REG(I2C_SLAVE2)
#define SLAVE3_I2C                    REG(I2C_SLAVE3)
#define SLAVE4_I2C                    REG(I2C_SLAVE4)
#define SLAVE5_I2C                    REG(I2C_SLAVE5)

void i2c_master_write_byte(BYTE address, BYTE reg, BYTE data);
void i2c_master_write_half_word(BYTE address, BYTE reg, BYTE data0, BYTE data1);
void i2c_master_write_word(BYTE address, BYTE reg, BYTE data0, BYTE data1, BYTE data2, BYTE data3);

void i2c_master_write_three_bytes(BYTE address, BYTE reg, BYTE data0, BYTE data1, BYTE data2);
void i2c_master_write_five_bytes(BYTE address, BYTE reg, BYTE data0, BYTE data1, BYTE data2, BYTE data3, BYTE data4);
void i2c_master_write_six_bytes(BYTE address, BYTE reg, BYTE data0, BYTE data1, BYTE data2, BYTE data3, BYTE data4, BYTE data5);

void i2c_master_read(BYTE address, BYTE reg);

void i2c_enable();
void i2c_disable();

void i2c_master_mode_enable();
void i2c_master_mode_disable();

void i2c_general_call_enable();
void i2c_general_call_disable();

void i2c_interactive_receive_mode_enabled();
void i2c_interactive_receive_mode_disabled();

void i2c_data_ack();
void i2c_data_nack();

void i2c_drive_scl_low();
void i2c_release_scl();

void i2c_drive_sda_low();
void i2c_release_sda();

BYTE i2c_scl_status();
BYTE i2c_sda_status();

void i2c_software_output_enabled();
void i2c_software_output_disabled();

BYTE i2c_read_bit();

void i2c_scl_stretch_enabled();
void i2c_scl_stretch_disabled();

void i2c_open_drain_operation();
void i2c_push_pull_operation();

void i2c_hs_mode_enabled();
void i2c_hs_mode_disabled();

BYTE i2c_bus_status();

BYTE i2c_rx_empty();
BYTE i2c_rx_full();

BYTE i2c_tx_empty();
BYTE i2c_tx_full();

BYTE i2c_scl_mode();

BYTE i2c_controller_status();

void i2c_clear_irq_transfer_done();
BYTE i2c_is_transfer_done_irq();

void i2c_clear_irq_interactive_receive();
BYTE i2c_is_interactive_receive_irq();

void i2c_clear_irq_general_call_address_match();
BYTE i2c_is_general_call_address_match_irq();

void i2c_clear_irq_slave_address_match();
BYTE i2c_is_slave_address_match_irq();

void i2c_clear_irq_rx_thd();
BYTE i2c_is_rx_thd_irq();

void i2c_clear_irq_tx_thd();
BYTE i2c_is_tx_thd_irq();

void i2c_clear_irq_stop();
BYTE i2c_is_stop_irq();

void i2c_clear_irq_address_ack();
BYTE i2c_is_address_ack_irq();

void i2c_clear_irq_arbitration_error();
BYTE i2c_is_arbitration_error_irq();

void i2c_clear_irq_timeout_error();
BYTE i2c_is_timeout_error_irq();

void i2c_clear_irq_address_nack_error();
BYTE i2c_is_address_nack_error_irq();

void i2c_clear_irq_data_nack_error();
BYTE i2c_is_data_nack_error_irq();

void i2c_clear_irq_do_not_respond_error();
BYTE i2c_is_do_not_respond_error_irq();

void i2c_clear_irq_start_error();
BYTE i2c_is_start_error_irq();

void i2c_clear_irq_stop_error();
BYTE i2c_is_stop_error_irq();

void i2c_clear_irq_transmit_lock_out_error();
BYTE i2c_is_transmit_lock_out_error_irq();

void i2c_clear_irq_slave0_address_match();
BYTE i2c_is_slave0_address_match_irq();

void i2c_clear_irq_slave1_address_match();
BYTE i2c_is_slave1_address_match_irq();

void i2c_clear_irq_slave2_address_match();
BYTE i2c_is_slave2_address_match_irq();

void i2c_clear_irq_slave3_address_match();
BYTE i2c_is_slave3_address_match_irq();

void i2c_clear_irq_slave4_address_match();
BYTE i2c_is_slave4_address_match_irq();

void i2c_clear_irq_slave5_address_match();
BYTE i2c_is_slave5_address_match_irq();

void i2c_clear_irq_slave_write_address_match();
BYTE i2c_is_slave_write_address_match_irq();

void i2c_clear_irq_slave_read_address_match();
BYTE i2c_is_slave_read_address_match_irq();

void i2c_transfer_done_irq_enabled();
void i2c_transfer_done_irq_disabled();

void i2c_interactive_receive_irq_enabled();
void i2c_interactive_receive_irq_disabled();

void i2c_general_call_irq_enabled();
void i2c_general_call_irq_disabled();

void i2c_slave_address_match_irq_enabled();
void i2c_slave_address_match_irq_disabled();

void i2c_rx_thd_irq_enabled();
void i2c_rx_thd_irq_disabled();

void i2c_tx_thd_irq_enabled();
void i2c_tx_thd_irq_disabled();

void i2c_stop_irq_enabled();
void i2c_stop_irq_disabled();

void i2c_address_ack_irq_enabled();
void i2c_address_ack_irq_disabled();

void i2c_arbitration_error_irq_enabled();
void i2c_arbitration_error_irq_disabled();

void i2c_timeout_error_irq_enabled();
void i2c_timeout_error_irq_disabled();

void i2c_address_nack_error_irq_enabled();
void i2c_address_nack_error_irq_disabled();

void i2c_data_nack_error_irq_enabled();
void i2c_data_nack_error_irq_disabled();

void i2c_do_not_respond_irq_enabled();
void i2c_do_not_respond_irq_disabled();

void i2c_start_error_irq_enabled();
void i2c_start_error_irq_disabled();

void i2c_stop_error_irq_enabled();
void i2c_stop_error_irq_disabled();

void i2c_transmit_lock_out_irq_enabled();
void i2c_transmit_lock_out_irq_disabled();

void i2c_slave0_address_match_irq_enabled();
void i2c_slave0_address_match_irq_disabled();

void i2c_slave1_address_match_irq_enabled();
void i2c_slave1_address_match_irq_disabled();

void i2c_slave2_address_match_irq_enabled();
void i2c_slave2_address_match_irq_disabled();

void i2c_slave3_address_match_irq_enabled();
void i2c_slave3_address_match_irq_disabled();

void i2c_slave4_address_match_irq_enabled();
void i2c_slave4_address_match_irq_disabled();

void i2c_slave5_address_match_irq_enabled();
void i2c_slave5_address_match_irq_disabled();

void i2c_slave_write_address_match_irq_enabled();
void i2c_slave_write_address_match_irq_disabled();

void i2c_slave_read_address_match_irq_enabled();
void i2c_slave_read_address_match_irq_disabled();

void i2c_clear_irq_receiver_overflow();
BYTE i2c_is_receiver_overflow_irq();

void i2c_clear_irq_transmit_underflow();
BYTE i2c_is_transmit_underflow_irq();

void i2c_clear_irq_start();
BYTE i2c_is_start_irq();

void i2c_receiver_overflow_irq_enabled();
void i2c_receiver_overflow_irq_disabled();

void i2c_transmit_underflow_irq_enabled();
void i2c_transmit_underflow_irq_disabled();

void i2c_start_irq_enabled();
void i2c_start_irq_disabled();

BYTE i2c_rx_fifo_lenght();
BYTE i2c_tx_fifo_lenght();

void i2c_rx_not_empty_do_not_respond();
void i2c_rx_not_empty_respond();

void i2c_rx_fifo_flush();

void i2c_set_rx_fifo_thd(BYTE thd);

void i2c_set_rx_bytes_count(BYTE bytes);

BYTE i2c_rx_fifo_bytes_count();

void i2c_tx_fifo_preload_enabled();
void i2c_tx_fifo_preload_disabled();

void i2c_tx_ready_manual_mode_enabled();
void i2c_tx_ready_manual_mode_disabled();

void i2c_tx_fifo_auto_flush_general_call_address_match_disable();
void i2c_tx_fifo_auto_flush_general_call_address_match_enable();

void i2c_tx_fifo_auto_flush_slave_address_match_write_disable();
void i2c_tx_fifo_auto_flush_slave_address_match_write_enable();

void i2c_tx_fifo_auto_flush_slave_address_match_read_disable();
void i2c_tx_fifo_auto_flush_slave_address_match_read_enable();

void i2c_tx_fifo_auto_flush_nack_end_slave_transmit_disable();
void i2c_tx_fifo_auto_flush_nack_end_slave_transmit_enable();

void i2c_tx_fifo_flush();

void i2c_set_tx_fifo_thd(BYTE thd);

void i2c_tx_fifo_ready_set();
BYTE i2c_tx_fifo_ready();

void i2c_tx_last_hold_scl_low();
void i2c_tx_last_end_transaction();

BYTE i2c_tx_fifo_bytes_count();

void i2c_write_data(BYTE data);

BYTE i2c_read_data();

void i2c_master_start_transaction();

void i2c_master_repeated_start();

void i2c_master_stop();
BYTE i2c_is_master_stop();

void i2c_slave_address_seven();
void i2c_slave_address_ten();

void i2c_master_code(BYTE code);

void i2c_master_ack_ignore_enabled();
void i2c_master_ack_ignore_disabled();

void i2c_clock_low(HALF_WORD period);

void i2c_clock_high(HALF_WORD period);

void i2c_hs_mode_clock_low(HALF_WORD period);

void i2c_hs_mode_clock_high(HALF_WORD period);

void i2c_timeout_disable();
void i2c_set_timeout(HALF_WORD timeout);

void i2c_dma_tx_channel_enabled();
void i2c_dma_tx_channel_disabled();

void i2c_dma_rx_channel_enabled();
void i2c_dma_rx_channel_disabled();

void i2c_slave0_address(HALF_WORD address);

void i2c_slave0_address_enabled();
void i2c_slave0_address_disabled();

void i2c_slave0_extended_address_enabled();
void i2c_slave0_extended_address_disabled();

void i2c_slave1_address(HALF_WORD address);

void i2c_slave1_address_enabled();
void i2c_slave1_address_disabled();

void i2c_slave1_extended_address_enabled();
void i2c_slave1_extended_address_disabled();

void i2c_slave2_address(HALF_WORD address);

void i2c_slave2_address_enabled();
void i2c_slave2_address_disabled();

void i2c_slave2_extended_address_enabled();
void i2c_slave2_extended_address_disabled();

void i2c_slave3_address(HALF_WORD address);

void i2c_slave3_address_enabled();
void i2c_slave3_address_disabled();

void i2c_slave3_extended_address_enabled();
void i2c_slave3_extended_address_disabled();

void i2c_slave4_address(HALF_WORD address);

void i2c_slave4_address_enabled();
void i2c_slave4_address_disabled();

void i2c_slave4_extended_address_enabled();
void i2c_slave4_extended_address_disabled();

void i2c_slave5_address(HALF_WORD address);

void i2c_slave5_address_enabled();
void i2c_slave5_address_disabled();

void i2c_slave5_extended_address_enabled();
void i2c_slave5_extended_address_disabled();

#endif /* I2C_H_ */
