// Copyright 2017 ETH Zurich and University of Bologna.
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 0.51 (the); you may not use this file except in
// compliance with the License.  You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-0.51. Unless required by applicable law
// or agreed to in writing, software, hardware and materials distributed under
// this License is distributed on an AS IS BASIS, WITHOUT WARRANTIES OR
// CONDITIONS OF ANY KIND, either express or implied. See the License for the
// specific language governing permissions and limitations under the License.

#include "pulpino.h"

#define EXCEPTION_STACK_SIZE 96


/* ========================================================= [ entry ] === */
    .section .stack
    .align 3
#ifdef __STACK_SIZE
    .equ    Stack_Size, __STACK_SIZE
#else
    .equ    Stack_Size, 0x00001000
#endif
    .globl    __StackTop
    .globl    __StackLimit
__StackLimit:
    .space    Stack_Size
    .size __StackLimit, . - __StackLimit
__StackTop:
    .size __StackTop, . - __StackTop

    .section .heap
    .align 3
#ifdef __HEAP_SIZE
    .equ    Heap_Size, __HEAP_SIZE
#else
    .equ    Heap_Size, 0x00000C00
#endif
    .globl    __HeapBase
    .globl    __HeapLimit
__HeapBase:
    .if    Heap_Size
    .space    Heap_Size
    .endif
    .size __HeapBase, . - __HeapBase
__HeapLimit:
    .size __HeapLimit, . - __HeapLimit
/* HACK NI - update this section to use def_irq_handler instead of just listing the address of IRQ handler */
/* MLB - well just stick jumps here and define the handler as weak later */
    .section .isr_vector
    .align 2
    .globl __isr_vector
__isr_vector:
    j   PowerFail_Handler            /* IRQ 00 */
    j   BusError_Handler             /* IRQ 01 */
    j   IRQ_02_Handler
    j   IRQ_03_Handler
    j   IRQ_04_Handler
    j   IRQ_05_Handler
    j   IRQ_06_Handler
    j   IRQ_07_Handler
    j   IRQ_08_Handler
    j   IRQ_09_Handler
    j   IRQ_10_Handler
    j   IRQ_11_Handler
    j   IRQ_12_Handler
    j   IRQ_13_Handler
    j   IRQ_14_Handler
    j   IRQ_15_Handler
    j   IRQ_16_Handler
    j   IRQ_17_Handler
    j   IRQ_18_Handler
    j   IRQ_19_Handler
    j   IRQ_20_Handler
    j   IRQ_21_Handler
    j   IRQ_22_Handler
    j   IRQ_23_Handler
    j   IRQ_24_Handler
    j   IRQ_25_Handler
    j   IRQ_26_Handler
    j   IRQ_27_Handler
    j   IRQ_28_Handler
    j   IRQ_29_Handler
    j   IRQ_30_Handler
    j   IRQ_31_Handler
    j   Reset_Handler
    j   illegal_insn_handler
    j   ecall_insn_handler

  .section .text
    .align 2
    .globl   Reset_Handler
    .type    Reset_Handler, %function
Reset_Handler:

  /* set 0 in mtvec (base for IVT) */
  csrrw x0, mtvec, x0

  /* Initialize Stack Pointer */
.option push
.option norelax
  la   sp, __StackTop
.option pop

  /* Initialize Global Pointer */
.option push
.option norelax
  la gp, __global_pointer$
.option pop

  /* PreInit runs before any RAM initialization. Example usage: SPID setup, etc */
  call PreInit
  bnez a0, SKIPRAMINIT

_start:
  .global _start

/*     Loop to copy data from read only memory to RAM. The ranges
 *      of copy from/to are specified by following symbols evaluated in
 *      linker script.
 *      __load_data: Where data sections are saved.
 *      _data /_edata: RAM address range that data should be
 *      copied to. Both must be aligned to 4 bytes boundary.  */

szero_bss:
  la t0, _sbss
  la t1, _esbss
  bge t0, t1, szero_bss_loop_end

szero_bss_loop:

  sw x0, 0(t0)
  addi t0, t0, 4
  blt t0, t1, szero_bss_loop

szero_bss_loop_end:

  la t0, __load_sdata
  la t1, _sdata
  la t2, _esdata

  bge t1, t2, sinit_loop_end

/* MLB - There must be a way to speed this up... */
sinit_loop:
  lw  t3, 0(t0)
  sw   t3, 0(t1)
  addi t0, t0, 4
  addi t1, t1, 4
  blt t1, t2, sinit_loop
sinit_loop_end:

/* MLB - There must be a way to speed this up... */
  la t0, __load_data
  la t1, _data
  la t2, _edata

  bge t1, t2, init_loop_end

init_loop:
  lw  t3, 0(t0)
  sw   t3, 0(t1)
  addi t0, t0, 4
  addi t1, t1, 4
  blt t1, t2, init_loop
init_loop_end:

zero_bss:
  la t0, _bss
  la t1, _ebss
  bge t0, t1, zero_bss_loop_end

zero_bss_loop:

  sw x0, 0(t0)
  addi t0, t0, 4
  blt t0, t1, zero_bss_loop

zero_bss_loop_end:
SKIPRAMINIT:

  /* Perform System initialization after RAM initialization */
  call SystemInit

  /* Run global initialization functions */
  call    __libc_init_array

main_entry:

  /* jump to main program entry point */
  call main

.SPIN:
    /* spin if main ever returns. */
    jal .SPIN

/* =================================================== [ exceptions ] === */
/* This section has to be down here, since we have to disable rvc for it  */
/*    Macro to define default handlers. Default handler
 *    will be weak symbol and just dead loops. They can be
 *    overwritten by other handlers */
/* HACK NI - move up macro to be used in ISR vector table*/
    .macro    def_irq_handler    handler_name
    .align 2
    .weak    \handler_name
    .type    \handler_name, %function
\handler_name :
    jal    .
    .size    \handler_name, . - \handler_name
    .endm

    def_irq_handler PowerFail_Handler            /* IRQ 00 */
    def_irq_handler BusError_Handler             /* IRQ 01 */
    def_irq_handler IRQ_02_Handler
    def_irq_handler IRQ_03_Handler
    def_irq_handler IRQ_04_Handler
    def_irq_handler IRQ_05_Handler
    def_irq_handler IRQ_06_Handler
    def_irq_handler IRQ_07_Handler
    def_irq_handler IRQ_08_Handler
    def_irq_handler IRQ_09_Handler
    def_irq_handler IRQ_10_Handler
    def_irq_handler IRQ_11_Handler
    def_irq_handler IRQ_12_Handler
    def_irq_handler IRQ_13_Handler
    def_irq_handler IRQ_14_Handler
    def_irq_handler IRQ_15_Handler
    def_irq_handler IRQ_16_Handler
    def_irq_handler IRQ_17_Handler
    def_irq_handler IRQ_18_Handler
    def_irq_handler IRQ_19_Handler
    def_irq_handler IRQ_20_Handler
    def_irq_handler IRQ_21_Handler
    def_irq_handler IRQ_22_Handler
    def_irq_handler IRQ_23_Handler
    def_irq_handler IRQ_24_Handler
    def_irq_handler IRQ_25_Handler
    def_irq_handler IRQ_26_Handler
    def_irq_handler IRQ_27_Handler
    def_irq_handler IRQ_28_Handler
    def_irq_handler IRQ_29_Handler
    def_irq_handler IRQ_30_Handler
    def_irq_handler IRQ_31_Handler
    def_irq_handler illegal_insn_handler
    def_irq_handler ecall_insn_handler

  .global _init
  .global _fini
_init:
_fini:
  # These don't have to do anything since we use init_array/fini_array.
  ret 
