/*
 * lpuart.c
 *
 *  Created on: 3 lug 2020
 *      Author: Federico.Pozzana
 */

#include "uart.h"
#include <string.h>

char instruction[INSTRUCTION_LENGHT];
BYTE instruction_ready;


void uart_parity_enable(){
	CTRL_UART |= (1 << 4);
}

void uart_parity_disable(){
	CTRL_UART &= ~(1 << 4);
}

void uart_even_parity(){
	CTRL_UART &= ~(1 << 5);
}

void uart_odd_parity(){
	CTRL_UART |= (1 << 5);
}

void uart_zero_parity(){
	CTRL_UART |= (1 << 6);
}

void uart_one_parity(){
	CTRL_UART &= ~(1 << 6);
}


/* UART PARITY CONFIGURATION
enable == 0 => parity disabled
enable == 1 => parity enabled
default => parity enabled
type == 0 => even parity
type == 1 => odd parity
default => odd parity
value == 0 => zero's parity
value == 1 => one's parity
default => one's parity  */
void uart_parity_config(BYTE enable, BYTE type, BYTE value){
	switch(enable)
	{
	case 0 : uart_parity_disable();
		     break;
	case 1 : uart_parity_enable();
	 	 	 break;
	default : uart_parity_enable();
	 	      break;
	}

	switch(type)
	{
	case 0 : uart_even_parity();
	         break;
	case 1 : uart_odd_parity();
	         break;
	default : uart_odd_parity();
		 	  break;
	}

	switch(value)
	{
	case 0 : uart_zero_parity();
	         break;
	case 1 : uart_one_parity();
	         break;
	default : uart_one_parity();
			  break;
	}
}

/* UART CHARACTER SIZE CONFIGURATION
size == 0 => size = 5
size == 1 => size = 6
size == 2 => size = 7
size == 3 => size = 8
default => size = 8 */
void uart_char_size(WORD size){
	switch(size)
	{
	case 0 : CTRL_UART = (CTRL_UART & ~(1 << 10)) | (CTRL_UART & ~(1 << 11));
	         break;
	case 1 : CTRL_UART = (CTRL_UART | (1 << 10)) | (CTRL_UART & ~(1 << 11));
		     break;
	case 2 : CTRL_UART = (CTRL_UART & ~(1 << 10)) | (CTRL_UART | (1 << 11));
		     break;
	case 3 : CTRL_UART = (CTRL_UART | (1 << 10)) | (CTRL_UART | (1 << 11));
		     break;
	default : CTRL_UART = (CTRL_UART | (1 << 10)) | (CTRL_UART | (1 << 11));
              break;
	}
}

/* UART STOP BIT CONFIGURATION
stop == 0 => 1 stop bit
stop == 1 => 2 stop bits (1.5 for 5 bit char size mode)
default => 1 stop bit */
void uart_stop_bit(BYTE stop){
	switch(stop)
	{
	case 0 : CTRL_UART &= ~(1 << 12);
	         break;
	case 1 : CTRL_UART |= (1 << 12);
		     break;
	default : CTRL_UART &= ~(1 << 12);
              break;
	}
}

/* UART BAUD CLOCK ENABLE/DISABLE
enable == 0 => baud clock disabled
enable == 1 => baud clock enabled
default => baud clock enabled */
void uart_baud_clk_enable(BYTE enable){
	switch(enable)
	{
	case 0 : CTRL_UART &= ~(1 << 15);
	         break;
	case 1 : CTRL_UART |= (1 << 15);
		     break;
	default : CTRL_UART |= (1 << 15);
              break;
	}
}

/* returning BAUD CLK READY */
BYTE uart_baud_clk_ready(){
	return ((CTRL_UART & ( 1 << 19 )) >> 19);
}

/* set divisor for baudrate setting */
void set_baudrate_divisor(WORD divisor){
	CKDIV_UART |= (divisor & 0x000fffff);
}

void osr_zero_set(){
	OSR_UART |= (1 << 0);
}

void osr_zero_clear(){
	OSR_UART &= ~(1 << 0);
}

void osr_one_set(){
	OSR_UART |= (1 << 1);
}

void osr_one_clear(){
	OSR_UART &= ~(1 << 1);
}

void osr_two_set(){
	OSR_UART |= (1 << 2);
}

void osr_two_clear(){
	OSR_UART &= ~(1 << 2);
}

/* send to UART transaction */
void send_uart(BYTE data){
	WORD data_mask = DATA_UART & ~0xff;
	DATA_UART = data_mask | data;
}

/* receive from UART transaction */
BYTE recieve_uart(){
	return (DATA_UART & 0x000000ff);
}

void enable_hardware_control(){
	CTRL_UART |= (1 << 13);
}

void disable_hardware_control(){
	CTRL_UART &= ~(1 << 13);
}

/* return 0 => rx fifo not empty
   return 1 => rx fifo empty */
BYTE rx_fifo_empty(){
	return ((STATUS_UART & ( 1 << 4 )) >> 4);
}

/* return 0 => rx fifo not full
   return 1 => rx fifo full */
BYTE rx_fifo_full(){
	return ((STATUS_UART & ( 1 << 5 )) >> 5);
}

/* return 0 => tx fifo not empty
   return 1 => tx fifo empty */
BYTE tx_fifo_empty(){
	return ((STATUS_UART & ( 1 << 6 )) >> 6);
}

/* return 0 => tx fifo not full
   return 1 => tx fifo full */
BYTE tx_fifo_full(){
	return ((STATUS_UART & ( 1 << 7 )) >> 7);
}

/* return 0 => tx fifo not busy
   return 1 => tx fifo busy */
BYTE tx_fifo_busy(){
	return ((STATUS_UART & ( 1 << 0 )) >> 0);
}

/* return 0 => rx fifo not busy
   return 1 => rx fifo busy */
BYTE rx_fifo_busy(){
	return ((STATUS_UART & ( 1 << 1 )) >> 1);
}

/*set the rx fifo threshold */
void set_rx_fifo_thd(BYTE threshold){
	CTRL_UART &= 0xfffffff0;
	CTRL_UART |= (threshold & 0x0f);
}

/*enable threshold generated interrupt */
void enable_int_rx_thd(){
	INTEN_UART |= (1 << 4);
}

/*disable threshold generated interrupt */
void disable_int_rx_thd(){
	INTEN_UART &= ~(1 << 4);
}

void enable_int_rx_frame_err(){
	INTEN_UART |= (1 << 0);
}

void disable_int_rx_frame_err(){
	INTEN_UART &= ~(1 << 0);
}

void enable_int_rx_parity_err(){
	INTEN_UART |= (1 << 1);
}

void disable_int_rx_parity_err(){
	INTEN_UART &= ~(1 << 1);
}

void enable_int_cts_change(){
	INTEN_UART |= (1 << 2);
}

void disable_int_cts_change(){
	INTEN_UART &= ~(1 << 2);
}

void enable_int_overrun(){
	INTEN_UART |= (1 << 3);
}

void disable_int_overrun(){
	INTEN_UART &= ~(1 << 3);
}

void enable_int_tx_fifo_one_left(){
	INTEN_UART |= (1 << 5);
}

void disable_int_tx_fifo_one_left(){
	INTEN_UART &= ~(1 << 5);
}

void enable_int_tx_fifo_half_empty(){
	INTEN_UART |= (1 << 6);
}

void disable_int_tx_fifo_half_empty(){
	INTEN_UART &= ~(1 << 6);
}

/*clear interrupt rx threshold reached
  write 1 to clear */
void clear_rx_fifo_thd_int(){
	INTFL_UART |= (1 << 4);
}

/* return 1 if interrupt was
   generated by rx fifo reaching threshold  */
BYTE is_rx_fifo_thd(){
	return ((INTFL_UART & ( 1 << 4 )) >> 4);
}

void clear_rx_frame_err_int(){
	INTFL_UART |= (1 << 0);
}

void clear_rx_parity_err_int(){
	INTFL_UART |= (1 << 1);
}

void clear_cts_change_int(){
	INTFL_UART |= (1 << 2);
}

void clear_overrun_int(){
	INTFL_UART |= (1 << 3);
}

void clear_tx_fifo_one_left_int(){
	INTFL_UART |= (1 << 5);
}

void clear_tx_fifo_half_empty_int(){
	INTFL_UART |= (1 << 6);
}

void cts_sampling_enable(){
	CTRL_UART &= ~(1 << 7);
}

void cts_sampling_disable(){
	CTRL_UART |= (1 << 7);
}

void tx_flush(){
	CTRL_UART |= (1 << 8);
}

void rx_flush(){
	CTRL_UART |= (1 << 9);
}

/* condition to negate rts
   cond == 0 => negate when rx fifo is full
   cond == 1 -> negate when threshold is reached
   default => negate when ex fifo is full
 */
void rts_neg_cond(BYTE cond){
	switch(cond)
	{
		case 0 : CTRL_UART &= ~(1 << 14);
		         break;
		case 1 : CTRL_UART |= (1 << 14);
				 break;
		default : CTRL_UART &= ~(1 << 14);
        		  break;
	}
}

/* uart clock selection
   clock == 0 => APB clock
   clock == 1 => clock 1
   clock == 2 => clock 2
   clock == 3 => clock 3
   default => APB clock
 */
void clk_sel(BYTE clock){
	switch(clock)
	{
	   case 0 : CTRL_UART &= ~(1 << 16);
	   	   	    CTRL_UART &= ~(1 << 17);
                break;
	   case 1 : CTRL_UART |= (1 << 16);
	   	   	   	CTRL_UART &= ~(1 << 17);
	            break;
	   case 2 : CTRL_UART &= ~(1 << 16);
	   	   	   	CTRL_UART |= (1 << 17);
	            break;
	   case 3 : CTRL_UART |= (1 << 16);
	   	   	    CTRL_UART |= (1 << 17);
	            break;
	   default : CTRL_UART &= ~(1 << 16);
	   	   	     CTRL_UART &= ~(1 << 17);
	             break;
	}
}

void auto_gating_enable(){
	CTRL_UART |= (1 << 20);
}

void auto_gating_diable(){
	CTRL_UART &= ~(1 << 20);
}

void fractional_div_mode_enable(){
	CTRL_UART |= (1 << 21);
}

void fractional_div_mode_disable(){
	CTRL_UART &= ~(1 << 21);
}

void rx_dual_edge_enable(){
	CTRL_UART |= (1 << 22);
}

void rx_dual_edge_disable(){
	CTRL_UART &= ~(1 << 22);
}

void rx_tweak_enable(){
	CTRL_UART |= (1 << 23);
}

void rx_tweak_disable(){
	CTRL_UART &= ~(1 << 23);
}

void rx_tweak_value(BYTE tweak_value){
	WORD tweak_mask = CTRL_UART & ~0xff000000;
	CTRL_UART = tweak_mask | (tweak_value << 24);
}

BYTE rx_fifo_bytes(){
	return (STATUS_UART & (0xf << 8) >> 8);
}

BYTE tx_fifo_bytes(){
	return (STATUS_UART & (0xf << 12) >> 12);
}

BYTE tx_fifo_next_data(){
	return (TXFIFO_UART & 0x000000ff);
}

/* get current value of cts signal */
BYTE get_cts(){
	return (PNR_UART & 0x00000001);
}

/* set value of rts signal */
void set_rst(BYTE value){
	switch(value)
	{
		case 0 : PNR_UART &= ~(1 << 1);
				 break;
		case 1 : PNR_UART |= (1 << 1);
				 break;
		default : break;
	}
}

/* return parity error flag */
BYTE parity_err_flag(){
	return (DATA_UART & (0x1 << 8) >> 8);
}

void wkup_enable_rx_fifo_not_empty(){
	WKEN_UART |= (1 << 0);
}

void wkup_disable_rx_fifo_not_empty(){
	WKEN_UART &= ~(1 << 0);
}

BYTE wkup_flag_rx_fifo_not_empty(){
	return (WKEN_UART & (0x1 << 0) >> 0);
}

void wkup_enable_rx_fifo_full(){
	WKEN_UART |= (1 << 1);
}

void wkup_disable_rx_fifo_full(){
	WKEN_UART &= ~(1 << 1);
}

BYTE wkup_flag_rx_fifo_full(){
	return (WKEN_UART & (0x1 << 1) >> 1);
}

void wkup_enable_rx_fifo_thd(){
	WKEN_UART |= (1 << 2);
}

void wkup_disable_rx_fifo_thd(){
	WKEN_UART &= ~(1 << 2);
}

BYTE wkup_flag_rx_fifo_thd(){
	return (WKEN_UART & (0x1 << 2) >> 2);
}

void uart_printf(char *array){
	unsigned int n = strlen(array);
	for(int i = 0; i < n; i++){
		send_uart(array[i]);
		while(tx_fifo_full()){}
	}
	send_uart(10);
	while(tx_fifo_full()){}
	send_uart(13);
	while(tx_fifo_full()){}
}

void uart_scanf(){
	enable_int_rx_thd();
	while(!instruction_ready){};
	disable_int_rx_thd();
	instruction_ready = 0;
	uart_printf(instruction);
	for(int j = 0; j < INSTRUCTION_LENGHT; j++){
		instruction[j] = ' ';
	}
}
