/*
 * cordic.h
 *
 *  Created on: 31 ago 2020
 *      Author: Federico.Pozzana
 */

#ifndef CORDIC_H_
#define CORDIC_H_

#include "pulpino.h"

#define CORDIC_CTRL                  ( CORDIC_BASE_ADDR + 0x0000 )
#define CORDIC_STAT                  ( CORDIC_BASE_ADDR + 0x0004 )
#define CORDIC_X_DATA                ( CORDIC_BASE_ADDR + 0x0008 )
#define CORDIC_Y_DATA                ( CORDIC_BASE_ADDR + 0x000C )
#define CORDIC_Z_DATA                ( CORDIC_BASE_ADDR + 0x0010 )
#define CORDIC_COS                   ( CORDIC_BASE_ADDR + 0x0014 )
#define CORDIC_SIN                   ( CORDIC_BASE_ADDR + 0x0018 )

#define CTRL_CORDIC                  REG(CORDIC_CTRL)
#define STAT_CORDIC                  REG(CORDIC_STAT)
#define X_DATA_CORDIC                REG(CORDIC_X_DATA)
#define Y_DATA_CORDIC                REG(CORDIC_Y_DATA)
#define Z_DATA_CORDIC                REG(CORDIC_Z_DATA)
#define COS_CORDIC                   REG(CORDIC_COS)
#define SIN_CORDIC                   REG(CORDIC_SIN)

WORD cordic_polling();
void cordic_clear();

void cordic_x_data_byte_write(sBYTE data);
void cordic_y_data_byte_write(sBYTE data);
void cordic_z_data_byte_write(sBYTE data);

void cordic_x_data_half_word_write(sHALF_WORD data);
void cordic_y_data_half_word_write(sHALF_WORD data);
void cordic_z_data_half_word_write(sHALF_WORD data);

void cordic_x_data_word_write(sWORD data);
void cordic_y_data_word_write(sWORD data);
void cordic_z_data_word_write(sWORD data);

sBYTE cordic_cos_byte_read();
sBYTE cordic_sin_byte_read();

sHALF_WORD cordic_cos_half_word_read();
sHALF_WORD cordic_sin_half_word_read();

sWORD cordic_cos_word_read();
sWORD cordic_sin_word_read();

void cordic_start_rot();
void cordic_start_vect();

float degrees_to_radians(float angle);

BYTE cordic_8_angle(float angle);
HALF_WORD cordic_16_angle(float angle);
WORD cordic_32_angle(float angle);

float cordic_8_sin(WORD angle);
float cordic_16_sin(WORD angle);
float cordic_32_sin(WORD angle);

float cordic_8_cos(WORD angle);
float cordic_16_cos(WORD angle);
float cordic_32_cos(WORD angle);

float cordic_sqrt_half_word(int data);

#endif /* CORDIC_H_ */
