/*
 * i2c.c
 *
 *  Created on: 23 lug 2020
 *      Author: Federico.Pozzana
 */

#include "i2c.h"

void i2c_master_write_byte(BYTE address, BYTE data){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	BYTE first = (address << 1);
	i2c_write_data(first);
	i2c_write_data(data);
	i2c_master_stop();
	i2c_master_start_transaction();
}

void i2c_master_write_half_word(BYTE address, BYTE data0, BYTE data1){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	BYTE first = (address << 1);
	i2c_write_data(first);
	i2c_write_data(data0);
	i2c_write_data(data1);
	i2c_master_stop();
	i2c_master_start_transaction();
}

void i2c_master_write_word(BYTE address, BYTE data0, BYTE data1, BYTE data2, BYTE data3){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	BYTE first = (address << 1);
	i2c_write_data(first);
	i2c_write_data(data0);
	i2c_write_data(data1);
	i2c_write_data(data2);
	i2c_write_data(data3);
	i2c_master_stop();
	i2c_master_start_transaction();
}

/*
void i2c_master_read_byte_first_first(BYTE address, BYTE reg){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	//while(i2c_tx_fifo_bytes_count()){}
	//while(i2c_is_master_stop()){}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	BYTE first = (address << 1);
	i2c_write_data(first);
	//i2c_write_data(reg);
	i2c_master_repeated_start();
	i2c_master_start_transaction();
}

void i2c_master_read_byte_first_second(BYTE address){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	//while(i2c_tx_fifo_bytes_count()){}
	//while(i2c_is_master_stop()){}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	while(!i2c_is_transfer_done_irq()){}
	i2c_clear_irq_transfer_done();
	i2c_tx_fifo_flush();
	BYTE first = (address << 1);
	i2c_write_data(first | 0x01);
	i2c_master_stop();
	//i2c_master_repeated_start();
	i2c_master_start_transaction();
}
*/

void i2c_master_read(BYTE address){
	if(i2c_is_transmit_lock_out_error_irq()){
		i2c_clear_irq_transmit_lock_out_error();
	}
	if(i2c_is_master_stop()){
		while(!i2c_is_transfer_done_irq()){}
		i2c_clear_irq_transfer_done();
	}
	BYTE first = (address << 1);
	i2c_write_data(first);
	i2c_master_repeated_start();
	i2c_master_start_transaction();

	while(!i2c_is_transfer_done_irq()){}
	i2c_clear_irq_transfer_done();
	i2c_tx_fifo_flush();
	i2c_write_data(first | 0x01);
	i2c_master_stop();
	i2c_master_start_transaction();
}
void i2c_enable(){
	CTRL_I2C |= (1 << 0);
}

void i2c_disable(){
	CTRL_I2C &= ~(1 << 0);
}

void i2c_master_mode_enable(){
	CTRL_I2C |= (1 << 1);
}

void i2c_master_mode_disable(){
	CTRL_I2C &= ~(1 << 1);
}

void i2c_general_call_enable(){
	CTRL_I2C |= (1 << 2);
}

void i2c_general_call_disable(){
	CTRL_I2C &= ~(1 << 2);
}

void i2c_interactive_receive_mode_enabled(){
	CTRL_I2C |= (1 << 3);
}

void i2c_interactive_receive_mode_disabled(){
	CTRL_I2C &= ~(1 << 3);
}

void i2c_data_ack(){
	CTRL_I2C &= ~(1 << 4);
}

void i2c_data_nack(){
	CTRL_I2C |= (1 << 4);
}

void i2c_drive_scl_low(){
	CTRL_I2C &= ~(1 << 6);
}

void i2c_release_scl(){
	CTRL_I2C |= (1 << 6);
}

void i2c_drive_sda_low(){
	CTRL_I2C &= ~(1 << 7);
}

void i2c_release_sda(){
	CTRL_I2C |= (1 << 7);
}

BYTE i2c_scl_status(){
	return ((CTRL_I2C & (1 << 8)) >> 8);
}

BYTE i2c_sda_status(){
	return ((CTRL_I2C & (1 << 9)) >> 9);
}

void i2c_software_output_enabled(){
	CTRL_I2C |= (1 << 10);
}

void i2c_software_output_disabled(){
	CTRL_I2C &= ~(1 << 10);
}

BYTE i2c_read_bit(){
	return ((CTRL_I2C & (1 << 11)) >> 11);
}

void i2c_scl_stretch_enabled(){
	CTRL_I2C &= ~(1 << 12);
}

void i2c_scl_stretch_disabled(){
	CTRL_I2C |= (1 << 12);
}

void i2c_open_drain_operation(){
	CTRL_I2C &= ~(1 << 13);
}

void i2c_push_pull_operation(){
	CTRL_I2C |= (1 << 13);
}

void i2c_hs_mode_enabled(){
	CTRL_I2C |= (1 << 15);
}

void i2c_hs_mode_disabled(){
	CTRL_I2C &= ~(1 << 15);
}

BYTE i2c_bus_status(){
	return ((STAT_I2C & (1 << 0)) >> 0);
}

BYTE i2c_rx_empty(){
	return ((STAT_I2C & (1 << 1)) >> 1);
}

BYTE i2c_rx_full(){
	return ((STAT_I2C & (1 << 2)) >> 2);
}

BYTE i2c_tx_empty(){
	return ((STAT_I2C & (1 << 3)) >> 3);
}

BYTE i2c_tx_full(){
	return ((STAT_I2C & (1 << 4)) >> 4);
}

/* return 0 => not actively driving scl
   return 1 => actively driving scl */
BYTE i2c_scl_mode(){
	return ((STAT_I2C & (1 << 5)) >> 5);
}

/* return 0000 => idle
   return 0001 => master transmit address
   return 0010 => master receive address ack
   return 0011 => master transmit extended address
   return 0100 => master receive extended address ack
   return 0101 => slave receive address
   return 0110 => slave transmit address ack
   return 0111 => slave receive extended address
   return 1000 => slave transmit extended address ack
   return 1001 => transmit data (master or slave)
   return 1010 => receive data ack (master or slave)
   return 1011 => receive data (master or slave)
   return 1100 => transmit data ack (master or slave)
   return 1101 => nack stage (master or slave)
   return 1110 => ???
   return 1111 => bystander state */
BYTE i2c_controller_status(){
	return ((STAT_I2C & (0xf << 8)) >> 8);
}

void i2c_clear_irq_transfer_done(){
	INT0_I2C |= (1 << 0);
}

BYTE i2c_is_transfer_done_irq(){
	return ((INT0_I2C & (1 << 0)) >> 0);
}

void i2c_clear_irq_interactive_receive(){
	INT0_I2C |= (1 << 1);
}

BYTE i2c_is_interactive_receive_irq(){
	return ((INT0_I2C & (1 << 1)) >> 1);
}

void i2c_clear_irq_general_call_address_match(){
	INT0_I2C |= (1 << 2);
}

BYTE i2c_is_general_call_address_match_irq(){
	return ((INT0_I2C & (1 << 2)) >> 2);
}

void i2c_clear_irq_slave_address_match(){
	INT0_I2C |= (1 << 3);
}

BYTE i2c_is_slave_address_match_irq(){
	return ((INT0_I2C & (1 << 3)) >> 3);
}

void i2c_clear_irq_rx_thd(){
	INT0_I2C |= (1 << 4);
}

BYTE i2c_is_rx_thd_irq(){
	return ((INT0_I2C & (1 << 4)) >> 4);
}

void i2c_clear_irq_tx_thd(){
	INT0_I2C |= (1 << 5);
}

BYTE i2c_is_tx_thd_irq(){
	return ((INT0_I2C & (1 << 5)) >> 5);
}

void i2c_clear_irq_stop(){
	INT0_I2C |= (1 << 6);
}

BYTE i2c_is_stop_irq(){
	return ((INT0_I2C & (1 << 6)) >> 6);
}

void i2c_clear_irq_address_ack(){
	INT0_I2C |= (1 << 7);
}

BYTE i2c_is_address_ack_irq(){
	return ((INT0_I2C & (1 << 7)) >> 7);
}

void i2c_clear_irq_arbitration_error(){
	INT0_I2C |= (1 << 8);
}

BYTE i2c_is_arbitration_error_irq(){
	return ((INT0_I2C & (1 << 8)) >> 8);
}

void i2c_clear_irq_timeout_error(){
	INT0_I2C |= (1 << 9);
}

BYTE i2c_is_timeout_error_irq(){
	return ((INT0_I2C & (1 << 9)) >> 9);
}

void i2c_clear_irq_address_nack_error(){
	INT0_I2C |= (1 << 10);
}

BYTE i2c_is_address_nack_error_irq(){
	return ((INT0_I2C & (1 << 10)) >> 10);
}

void i2c_clear_irq_data_nack_error(){
	INT0_I2C |= (1 << 11);
}

BYTE i2c_is_data_nack_error_irq(){
	return ((INT0_I2C & (1 << 11)) >> 11);
}

void i2c_clear_irq_do_not_respond_error(){
	INT0_I2C |= (1 << 12);
}

BYTE i2c_is_do_not_respond_error_irq(){
	return ((INT0_I2C & (1 << 12)) >> 12);
}

void i2c_clear_irq_start_error(){
	INT0_I2C |= (1 << 13);
}

BYTE i2c_is_start_error_irq(){
	return ((INT0_I2C & (1 << 13)) >> 13);
}

void i2c_clear_irq_stop_error(){
	INT0_I2C |= (1 << 14);
}

BYTE i2c_is_stop_error_irq(){
	return ((INT0_I2C & (1 << 14)) >> 14);
}

void i2c_clear_irq_transmit_lock_out_error(){
	INT0_I2C |= (1 << 15);
}

BYTE i2c_is_transmit_lock_out_error_irq(){
	return ((INT0_I2C & (1 << 15)) >> 15);
}

void i2c_clear_irq_slave0_address_match(){
	INT0_I2C |= (1 << 16);
}

BYTE i2c_is_slave0_address_match_irq(){
	return ((INT0_I2C & (1 << 16)) >> 16);
}

void i2c_clear_irq_slave1_address_match(){
	INT0_I2C |= (1 << 17);
}

BYTE i2c_is_slave1_address_match_irq(){
	return ((INT0_I2C & (1 << 17)) >> 17);
}

void i2c_clear_irq_slave2_address_match(){
	INT0_I2C |= (1 << 18);
}

BYTE i2c_is_slave2_address_match_irq(){
	return ((INT0_I2C & (1 << 18)) >> 18);
}

void i2c_clear_irq_slave3_address_match(){
	INT0_I2C |= (1 << 19);
}

BYTE i2c_is_slave3_address_match_irq(){
	return ((INT0_I2C & (1 << 19)) >> 19);
}

void i2c_clear_irq_slave4_address_match(){
	INT0_I2C |= (1 << 20);
}

BYTE i2c_is_slave4_address_match_irq(){
	return ((INT0_I2C & (1 << 20)) >> 20);
}

void i2c_clear_irq_slave5_address_match(){
	INT0_I2C |= (1 << 21);
}

BYTE i2c_is_slave5_address_match_irq(){
	return ((INT0_I2C & (1 << 21)) >> 21);
}

void i2c_clear_irq_slave_write_address_match(){
	INT0_I2C |= (1 << 22);
}

BYTE i2c_is_slave_write_address_match_irq(){
	return ((INT0_I2C & (1 << 22)) >> 22);
}

void i2c_clear_irq_slave_read_address_match(){
	INT0_I2C |= (1 << 23);
}

BYTE i2c_is_slave_read_address_match_irq(){
	return ((INT0_I2C & (1 << 23)) >> 23);
}

void i2c_transfer_done_irq_enabled(){
	INTE0_I2C |= (1 << 0);
}

void i2c_transfer_done_irq_disabled(){
	INTE0_I2C &= ~(1 << 0);
}

void i2c_interactive_receive_irq_enabled(){
	INTE0_I2C |= (1 << 1);
}

void i2c_interactive_receive_irq_disabled(){
	INTE0_I2C &= ~(1 << 1);
}

void i2c_general_call_irq_enabled(){
	INTE0_I2C |= (1 << 2);
}

void i2c_general_call_irq_disabled(){
	INTE0_I2C &= ~(1 << 2);
}

void i2c_slave_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 3);
}

void i2c_slave_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 3);
}

void i2c_rx_thd_irq_enabled(){
	INTE0_I2C |= (1 << 4);
}

void i2c_rx_thd_irq_disabled(){
	INTE0_I2C &= ~(1 << 4);
}

void i2c_tx_thd_irq_enabled(){
	INTE0_I2C |= (1 << 5);
}

void i2c_tx_thd_irq_disabled(){
	INTE0_I2C &= ~(1 << 5);
}

void i2c_stop_irq_enabled(){
	INTE0_I2C |= (1 << 6);
}

void i2c_stop_irq_disabled(){
	INTE0_I2C &= ~(1 << 6);
}

void i2c_address_ack_irq_enabled(){
	INTE0_I2C |= (1 << 7);
}

void i2c_address_ack_irq_disabled(){
	INTE0_I2C &= ~(1 << 7);
}

void i2c_arbitration_error_irq_enabled(){
	INTE0_I2C |= (1 << 8);
}

void i2c_arbitration_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 8);
}

void i2c_timeout_error_irq_enabled(){
	INTE0_I2C |= (1 << 9);
}

void i2c_timeout_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 9);
}

void i2c_address_nack_error_irq_enabled(){
	INTE0_I2C |= (1 << 10);
}

void i2c_address_nack_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 10);
}

void i2c_data_nack_error_irq_enabled(){
	INTE0_I2C |= (1 << 11);
}

void i2c_data_nack_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 11);
}

void i2c_do_not_respond_irq_enabled(){
	INTE0_I2C |= (1 << 12);
}

void i2c_do_not_respond_irq_disabled(){
	INTE0_I2C &= ~(1 << 12);
}

void i2c_start_error_irq_enabled(){
	INTE0_I2C |= (1 << 13);
}

void i2c_start_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 13);
}

void i2c_stop_error_irq_enabled(){
	INTE0_I2C |= (1 << 14);
}

void i2c_stop_error_irq_disabled(){
	INTE0_I2C &= ~(1 << 14);
}

void i2c_transmit_lock_out_irq_enabled(){
	INTE0_I2C |= (1 << 15);
}

void i2c_transmit_lock_out_irq_disabled(){
	INTE0_I2C &= ~(1 << 15);
}

void i2c_slave0_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 16);
}

void i2c_slave0_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 16);
}

void i2c_slave1_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 17);
}

void i2c_slave1_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 17);
}

void i2c_slave2_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 18);
}

void i2c_slave2_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 18);
}

void i2c_slave3_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 19);
}

void i2c_slave3_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 19);
}

void i2c_slave4_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 20);
}

void i2c_slave4_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 20);
}

void i2c_slave5_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 21);
}

void i2c_slave5_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 21);
}

void i2c_slave_write_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 22);
}

void i2c_slave_write_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 22);
}

void i2c_slave_read_address_match_irq_enabled(){
	INTE0_I2C |= (1 << 23);
}

void i2c_slave_read_address_match_irq_disabled(){
	INTE0_I2C &= ~(1 << 23);
}

void i2c_clear_irq_receiver_overflow(){
	INT1_I2C |= (1 << 0);
}

BYTE i2c_is_receiver_overflow_irq(){
	return ((INT1_I2C & (1 << 0)) >> 0);
}

void i2c_clear_irq_transmit_underflow(){
	INT1_I2C |= (1 << 1);
}

BYTE i2c_is_transmit_underflow_irq(){
	return ((INT1_I2C & (1 << 1)) >> 1);
}

void i2c_clear_irq_start(){
	INT1_I2C |= (1 << 2);
}

BYTE i2c_is_start_irq(){
	return ((INT1_I2C & (1 << 2)) >> 2);
}

void i2c_receiver_overflow_irq_enabled(){
	INTE1_I2C |= (1 << 0);
}

void i2c_receiver_overflow_irq_disabled(){
	INTE1_I2C &= ~(1 << 0);
}

void i2c_transmit_underflow_irq_enabled(){
	INTE1_I2C |= (1 << 1);
}

void i2c_transmit_underflow_irq_disabled(){
	INTE1_I2C &= ~(1 << 1);
}

void i2c_start_irq_enabled(){
	INTE1_I2C |= (1 << 2);
}

void i2c_start_irq_disabled(){
	INTE1_I2C &= ~(1 << 2);
}

BYTE i2c_rx_fifo_lenght(){
	return ((I2C_FIFO_CFG & (0xff << 0)) >> 0);
}

BYTE i2c_tx_fifo_lenght(){
	return ((I2C_FIFO_CFG & (0xff << 8)) >> 8);
}

void i2c_rx_not_empty_do_not_respond(){
	RX_CFG_I2C |= (1 << 0);
}

void i2c_rx_not_empty_respond(){
	RX_CFG_I2C &= ~(1 << 0);
}

void i2c_rx_fifo_flush(){
	RX_CFG_I2C |= (1 << 7);
}

void i2c_set_rx_fifo_thd(BYTE thd){
	WORD mask = RX_CFG_I2C & ~0x00000f00;
	RX_CFG_I2C = ((thd & 0x0f) << 8) | mask;
}

void i2c_set_rx_bytes_count(BYTE bytes){
	WORD mask = RX_REG_I2C & ~0x000000ff;
	RX_REG_I2C = bytes | mask;
}

BYTE i2c_rx_fifo_bytes_count(){
	return ((RX_REG_I2C & (0xf << 8)) >> 8);
}

void i2c_tx_fifo_preload_enabled(){
	TX_CFG_I2C |= (1 << 0);
}

void i2c_tx_fifo_preload_disabled(){
	TX_CFG_I2C &= ~(1 << 0);
}

void i2c_tx_ready_manual_mode_enabled(){
	TX_CFG_I2C |= (1 << 1);
}

void i2c_tx_ready_manual_mode_disabled(){
	TX_CFG_I2C &= ~(1 << 1);
}

void i2c_tx_fifo_auto_flush_general_call_address_match_disable(){
	TX_CFG_I2C |= (1 << 2);
}

void i2c_tx_fifo_auto_flush_general_call_address_match_enable(){
	TX_CFG_I2C &= ~(1 << 2);
}

void i2c_tx_fifo_auto_flush_slave_address_match_write_disable(){
	TX_CFG_I2C |= (1 << 3);
}

void i2c_tx_fifo_auto_flush_slave_address_match_write_enable(){
	TX_CFG_I2C &= ~(1 << 3);
}

void i2c_tx_fifo_auto_flush_slave_address_match_read_disable(){
	TX_CFG_I2C |= (1 << 4);
}

void i2c_tx_fifo_auto_flush_slave_address_match_read_enable(){
	TX_CFG_I2C &= ~(1 << 4);
}

void i2c_tx_fifo_auto_flush_nack_end_slave_transmit_disable(){
	TX_CFG_I2C |= (1 << 5);
}

void i2c_tx_fifo_auto_flush_nack_end_slave_transmit_enable(){
	TX_CFG_I2C &= ~(1 << 5);
}

void i2c_tx_fifo_flush(){
	TX_CFG_I2C |= (1 << 7);
}

void i2c_set_tx_fifo_thd(BYTE thd){
	WORD mask = TX_CFG_I2C & ~0x00000f00;
	TX_CFG_I2C = ((thd & 0x0f) << 8) | mask;
}

void i2c_tx_fifo_ready_set(){
	TX_REG_I2C |= (1 << 0);
}

BYTE i2c_tx_fifo_ready(){
	return ((TX_REG_I2C & (1 << 0)) >> 0);
}

void i2c_tx_last_hold_scl_low(){
	TX_REG_I2C &= ~(1 << 1);
}

void i2c_tx_last_end_transaction(){
	TX_REG_I2C |= (1 << 1);
}

BYTE i2c_tx_fifo_bytes_count(){
	return ((TX_REG_I2C & (0xf << 8)) >> 8);
}

void i2c_write_data(BYTE data){
	DATA_I2C = data;
}

BYTE i2c_read_data(){
	return(DATA_I2C & 0xff);
}

void i2c_master_start_transaction(){
	MASTER_CTRL_I2C |= (1 << 0);
}

void i2c_master_repeated_start(){
	MASTER_CTRL_I2C |= (1 << 1);
}

void i2c_master_stop(){
	MASTER_CTRL_I2C |= (1 << 2);
}

BYTE i2c_is_master_stop(){
	return ((MASTER_CTRL_I2C & (1 << 2)) >> 2);
}

void i2c_slave_address_seven(){
	MASTER_CTRL_I2C &= ~(1 << 7);
}

void i2c_slave_address_ten(){
	MASTER_CTRL_I2C |= (1 << 7);
}

void i2c_master_code(BYTE code){
	WORD mask = MASTER_CTRL_I2C & ~0x00000700;
	MASTER_CTRL_I2C = ((code & 0x07) << 8) | mask;
}

void i2c_master_ack_ignore_enabled(){
	MASTER_CTRL_I2C |= (1 << 12);
}

void i2c_master_ack_ignore_disabled(){
	MASTER_CTRL_I2C &= ~(1 << 12);
}

void i2c_clock_low(HALF_WORD period){
	CLK_LOW_I2C |= period;
}

void i2c_clock_high(HALF_WORD period){
	CLK_HIGH_I2C |= period;
}

void i2c_hs_mode_clock_low(HALF_WORD period){
	HS_MODE_I2C |= period;
}

void i2c_hs_mode_clock_high(HALF_WORD period){
	HS_MODE_I2C |= (period << 16);
}

void i2c_timeout_disable(){
	TIMEOUT_I2C &= ~(0xffffffff);
}

void i2c_set_timeout(HALF_WORD timeout){
	TIMEOUT_I2C |= timeout;
}

void i2c_dma_tx_channel_enabled(){
	DMA_I2C |= (1 << 0);
}

void i2c_dma_tx_channel_disabled(){
	DMA_I2C &= ~(1 << 0);
}

void i2c_dma_rx_channel_enabled(){
	DMA_I2C |= (1 << 1);
}

void i2c_dma_rx_channel_disabled(){
	DMA_I2C &= ~(1 << 1);
}

void i2c_slave0_address(HALF_WORD address){
	WORD mask = SLAVE0_I2C & ~0x000003ff;
	SLAVE0_I2C = (address & 0x3ff) | mask;
}

void i2c_slave0_address_enabled(){
	SLAVE0_I2C &= ~(1 << 10);
}

void i2c_slave0_address_disabled(){
	SLAVE0_I2C |= (1 << 10);
}

void i2c_slave0_extended_address_enabled(){
	SLAVE0_I2C |= (1 << 15);
}

void i2c_slave0_extended_address_disabled(){
	SLAVE0_I2C &= ~(1 << 15);
}

void i2c_slave1_address(HALF_WORD address){
	WORD mask = SLAVE1_I2C & ~0x000003ff;
	SLAVE1_I2C = (address & 0x3ff) | mask;
}

void i2c_slave1_address_enabled(){
	SLAVE1_I2C &= ~(1 << 10);
}

void i2c_slave1_address_disabled(){
	SLAVE1_I2C |= (1 << 10);
}

void i2c_slave1_extended_address_enabled(){
	SLAVE1_I2C |= (1 << 15);
}

void i2c_slave1_extended_address_disabled(){
	SLAVE1_I2C &= ~(1 << 15);
}

void i2c_slave2_address(HALF_WORD address){
	WORD mask = SLAVE2_I2C & ~0x000003ff;
	SLAVE2_I2C = (address & 0x3ff) | mask;
}

void i2c_slave2_address_enabled(){
	SLAVE2_I2C &= ~(1 << 10);
}

void i2c_slave2_address_disabled(){
	SLAVE2_I2C |= (1 << 10);
}

void i2c_slave2_extended_address_enabled(){
	SLAVE2_I2C |= (1 << 15);
}

void i2c_slave2_extended_address_disabled(){
	SLAVE2_I2C &= ~(1 << 15);
}

void i2c_slave3_address(HALF_WORD address){
	WORD mask = SLAVE3_I2C & ~0x000003ff;
	SLAVE3_I2C = (address & 0x3ff) | mask;
}

void i2c_slave3_address_enabled(){
	SLAVE3_I2C &= ~(1 << 10);
}

void i2c_slave3_address_disabled(){
	SLAVE3_I2C |= (1 << 10);
}

void i2c_slave3_extended_address_enabled(){
	SLAVE3_I2C |= (1 << 15);
}

void i2c_slave3_extended_address_disabled(){
	SLAVE3_I2C &= ~(1 << 15);
}

void i2c_slave4_address(HALF_WORD address){
	WORD mask = SLAVE4_I2C & ~0x000003ff;
	SLAVE4_I2C = (address & 0x3ff) | mask;
}

void i2c_slave4_address_enabled(){
	SLAVE4_I2C &= ~(1 << 10);
}

void i2c_slave4_address_disabled(){
	SLAVE4_I2C |= (1 << 10);
}

void i2c_slave4_extended_address_enabled(){
	SLAVE4_I2C |= (1 << 15);
}

void i2c_slave4_extended_address_disabled(){
	SLAVE4_I2C &= ~(1 << 15);
}

void i2c_slave5_address(HALF_WORD address){
	WORD mask = SLAVE5_I2C & ~0x000003ff;
	SLAVE5_I2C = (address & 0x3ff) | mask;
}

void i2c_slave5_address_enabled(){
	SLAVE5_I2C &= ~(1 << 10);
}

void i2c_slave5_address_disabled(){
	SLAVE5_I2C |= (1 << 10);
}

void i2c_slave5_extended_address_enabled(){
	SLAVE5_I2C |= (1 << 15);
}

void i2c_slave5_extended_address_disabled(){
	SLAVE5_I2C &= ~(1 << 15);
}
