/*
 * spi.c
 *
 *  Created on: 15 lug 2020
 *      Author: Federico.Pozzana
 */

#include "spi.h"

void spi_push_byte(BYTE data){
	WORD data_mask = DATA_SPI & ~0xff;
	DATA_SPI = data_mask | data;
}


void spi_push_half_word(HALF_WORD data){
	WORD data_mask = DATA_SPI & ~0xff;
	DATA_SPI = data_mask | data;
}

void spi_push_word(WORD data){
	DATA_SPI = data;
}

BYTE spi_pull_byte(){
	return(DATA_SPI & 0xff);
}

HALF_WORD spi_pull_half_word(){
	return(DATA_SPI & 0xffff);
}

WORD spi_pull_word(){
	return(DATA_SPI);
}

void spi_enable(){
	CTRL1_SPI |= (1 << 0);
}

void spi_disable(){
	CTRL1_SPI &= ~(1 << 0);
}

void spi_master_mode(){
	CTRL1_SPI |= (1 << 1);
}

void spi_slave_mode(){
	CTRL1_SPI &= ~(1 << 1);
}

void spi_ss0_output(){
	CTRL1_SPI &= ~(1 << 4);
}

void spi_ss0_input(){
	CTRL1_SPI |= (1 << 4);
}

void spi_start_transaction(){
	CTRL1_SPI |= (1 << 5);
}

/* case 0 => SPI de-assert ss at the end of transaction
   case 1 => SPI do not de-assert ss at the end of transaction
   default => SPI de-assert ss at the end of transaction */
void spi_ss_ctrl(BYTE ctrl){
	switch(ctrl)
	{
		case 0 : CTRL1_SPI &= ~(1 << 8);
				 break;
		case 1 : CTRL1_SPI |= (1 << 8);
				 break;
		default : CTRL1_SPI &= ~(1 << 8);
		 	 	  break;
	}
}

/* depending on ss_num (0-7) slave x is selected
   toggle bit to b compliant with spi_ss_ctrl
   default ss0 is toggled */
void spi_ss_sel(BYTE ss_num){
	switch(ss_num)
	{
		case 0 : CTRL1_SPI |= (1 << 16);
				 break;
		case 1 : CTRL1_SPI |= (1 << 17);
				 break;
		case 2 : CTRL1_SPI |= (1 << 18);
				 break;
		case 3 : CTRL1_SPI |= (1 << 19);
				 break;
		case 4 : CTRL1_SPI |= (1 << 20);
				 break;
		case 5 : CTRL1_SPI |= (1 << 21);
				 break;
		case 6 : CTRL1_SPI |= (1 << 22);
				 break;
		case 7 : CTRL1_SPI |= (1 << 23);
				 break;
		default : CTRL1_SPI |= (1 << 16);
		 	 	  break;
	}
}

void spi_ss_desel(BYTE ss_num){
	switch(ss_num)
	{
		case 0 : CTRL1_SPI &= ~(1 << 16);
				 break;
		case 1 : CTRL1_SPI &= ~(1 << 17);
				 break;
		case 2 : CTRL1_SPI &= ~(1 << 18);
				 break;
		case 3 : CTRL1_SPI &= ~(1 << 19);
				 break;
		case 4 : CTRL1_SPI &= ~(1 << 20);
				 break;
		case 5 : CTRL1_SPI &= ~(1 << 21);
				 break;
		case 6 : CTRL1_SPI &= ~(1 << 22);
				 break;
		case 7 : CTRL1_SPI &= ~(1 << 23);
				 break;
		default : CTRL1_SPI &= ~(1 << 16);
		 	 	  break;
	}
}

/* number of characters to transmit for transaction */
void spi_tx_num_char(HALF_WORD num){
	WORD data_mask = CTRL2_SPI & ~0xffff;
	CTRL2_SPI = data_mask | num;
}

/* number of characters to receive for transaction */
void spi_rx_num_char(HALF_WORD num){
	WORD data_mask = CTRL2_SPI & ~0xffff0000;
	CTRL2_SPI = data_mask | (num << 16);
}

void spi_cpol_zero(){
	CTRL3_SPI &= ~(1 << 0);
}

void spi_cpol_one(){
	CTRL3_SPI |= (1 << 0);
}

void spi_cpha_zero(){
	CTRL3_SPI &= ~(1 << 1);
}

void spi_cpha_one(){
	CTRL3_SPI |= (1 << 1);
}

/* mode 0 => cpol cpha 0 0
   mode 1 => cpol cpha 0 1
   mode 2 => cpol cpha 1 0
   mode 3 => cpol cpha 1 1
   default => mode 0 */
void spi_mode(BYTE mode){
	switch(mode)
	{
		case 0 : spi_cpol_zero();
				 spi_cpha_zero();
				 break;
		case 1 : spi_cpol_zero();
		 	 	 spi_cpha_one();
				 break;
		case 2 : spi_cpol_one();
		 	 	 spi_cpha_zero();
				 break;
		case 3 : spi_cpol_one();
		 	 	 spi_cpha_one();
				 break;
		default : spi_cpol_zero();
		 	 	  spi_cpha_zero();
		 	 	  break;
		}
}

void spi_char_num_bits(BYTE num){
	WORD mask = num & 0xf;
	CTRL3_SPI |= (mask << 8);
}

void spi_three_wire(){
	CTRL3_SPI |= (1 << 15);
}

void spi_four_wire(){
	CTRL3_SPI &= ~(1 << 15);
}

/* slave select ss_num is active low */
void spi_ss_pol_low(BYTE ss_num){
	switch(ss_num)
	{
		case 0 : CTRL3_SPI &= ~(1 << 16);
				 break;
		case 1 : CTRL3_SPI &= ~(1 << 17);
				 break;
		case 2 : CTRL3_SPI &= ~(1 << 18);
				 break;
		case 3 : CTRL3_SPI &= ~(1 << 19);
				 break;
		case 4 : CTRL3_SPI &= ~(1 << 20);
				 break;
		case 5 : CTRL3_SPI &= ~(1 << 21);
				 break;
		case 6 : CTRL3_SPI &= ~(1 << 22);
				 break;
		case 7 : CTRL3_SPI &= ~(1 << 23);
				 break;
		default : CTRL3_SPI &= ~(1 << 16);
		 	 	  break;
	}
}

/* slave select ss_num is active high */
void spi_ss_pol_high(BYTE ss_num){
	switch(ss_num)
	{
		case 0 : CTRL3_SPI |= (1 << 16);
				 break;
		case 1 : CTRL3_SPI |= (1 << 17);
				 break;
		case 2 : CTRL3_SPI |= (1 << 18);
				 break;
		case 3 : CTRL3_SPI |= (1 << 19);
				 break;
		case 4 : CTRL3_SPI |= (1 << 20);
				 break;
		case 5 : CTRL3_SPI |= (1 << 21);
				 break;
		case 6 : CTRL3_SPI |= (1 << 22);
				 break;
		case 7 : CTRL3_SPI |= (1 << 23);
				 break;
		default : CTRL3_SPI |= (1 << 16);
		 	 	  break;
	}
}

void spi_ss_delay1(BYTE num){
	WORD mask = CTRL4_SPI & ~0x000000ff;
	CTRL4_SPI = num | mask;
}

void spi_ss_delay2(BYTE num){
	WORD mask = CTRL4_SPI & ~0x0000ff00;
	CTRL4_SPI = (num << 8) | mask;
}

void spi_ss_inactive_delay(BYTE num){
	WORD mask = CTRL4_SPI & ~0x00ff0000;
	CTRL4_SPI = (num << 16) | mask;
}

void spi_duty_low_ctrl(BYTE ctrl){
	WORD mask = BRG_CTRL_SPI & ~0x000000ff;
	BRG_CTRL_SPI = ctrl | mask;
}

void spi_duty_high_ctrl(BYTE ctrl){
	WORD mask = BRG_CTRL_SPI & ~0x0000ff00;
	BRG_CTRL_SPI = (ctrl << 8) | mask;
}

void spi_duty_scale(BYTE factor){
	WORD mask = BRG_CTRL_SPI & ~0x00ff0000;
	BRG_CTRL_SPI = (factor << 16) | mask;
}

void spi_clear_irq_tx_thd(){
	IRQ_SPI |= (1 << 0);
}

void spi_clear_irq_tx_empty(){
	IRQ_SPI |= (1 << 1);
}

void spi_clear_irq_rx_thd(){
	IRQ_SPI |= (1 << 2);
}

void spi_clear_irq_rx_full(){
	IRQ_SPI |= (1 << 3);
}

void spi_clear_irq_ss_asserted(){
	IRQ_SPI |= (1 << 4);
}

void spi_clear_irq_ss_deasserted(){
	IRQ_SPI |= (1 << 5);
}

void spi_clear_irq_multi_master_fail(){
	IRQ_SPI |= (1 << 8);
}

void spi_clear_irq_slave_abort(){
	IRQ_SPI |= (1 << 9);
}

void spi_clear_irq_master_done(){
	IRQ_SPI |= (1 << 11);
}

void spi_clear_irq_tx_overrun(){
	IRQ_SPI |= (1 << 12);
}

void spi_clear_irq_tx_underrun(){
	IRQ_SPI |= (1 << 13);
}

void spi_clear_irq_rx_overrun(){
	IRQ_SPI |= (1 << 14);
}

void spi_clear_irq_rx_underrun(){
	IRQ_SPI |= (1 << 15);
}

void spi_tx_thd_enable_irq(){
	IRQE_SPI |= (1 << 0);
}
void spi_tx_thd_disable_irq(){
	IRQE_SPI &= ~(1 << 0);
}

void spi_tx_empty_enable_irq(){
	IRQE_SPI |= (1 << 1);
}
void spi_tx_empty_disable_irq(){
	IRQE_SPI &= ~(1 << 1);
}

void spi_rx_thd_enable_irq(){
	IRQE_SPI |= (1 << 2);
}
void spi_rx_thd_disable_irq(){
	IRQE_SPI &= ~(1 << 2);
}

void spi_rx_full_enable_irq(){
	IRQE_SPI |= (1 << 3);
}
void spi_rx_full_disable_irq(){
	IRQE_SPI &= ~(1 << 3);
}

void spi_ss_asserted_enable_irq(){
	IRQE_SPI |= (1 << 4);
}

void spi_ss_asserted_disable_irq(){
	IRQE_SPI &= ~(1 << 4);
}

void spi_ss_deasserted_enable_irq(){
	IRQE_SPI |= (1 << 5);
}

void spi_ss_deasserted_disable_irq(){
	IRQE_SPI &= ~(1 << 5);
}

void spi_multi_master_fault_enable_irq(){
	IRQE_SPI |= (1 << 8);
}
void spi_multi_master_fault_disable_irq(){
	IRQE_SPI &= ~(1 << 8);
}

void spi_slave_abort_enable_irq(){
	IRQE_SPI |= (1 << 9);
}

void spi_slave_abort_disable_irq(){
	IRQE_SPI &= ~(1 << 9);
}

void spi_master_done_enable_irq(){
	IRQE_SPI |= (1 << 11);
}

void spi_master_done_disable_irq(){
	IRQE_SPI &= ~(1 << 11);
}

void spi_tx_overrun_enable_irq(){
	IRQE_SPI |= (1 << 12);
}

void spi_tx_overrun_disable_irq(){
	IRQE_SPI &= ~(1 << 12);
}

void spi_tx_underrun_enable_irq(){
	IRQE_SPI |= (1 << 13);
}

void spi_tx_underrun_disable_irq(){
	IRQE_SPI &= ~(1 << 13);
}

void spi_rx_overrun_enable_irq(){
	IRQE_SPI |= (1 << 14);
}

void spi_rx_overrun_disable_irq(){
	IRQE_SPI &= ~(1 << 14);
}

void spi_rx_underrun_enable_irq(){
	IRQE_SPI |= (1 << 15);
}

void spi_rx_underrun_disable_irq(){
	IRQE_SPI &= ~(1 << 15);
}

/* return spi status
   1 => SPI active
   0 => SPI inactive */
BYTE spi_status(){
	return ((STAT_SPI & ( 1 << 0 )) >> 0);
}

void spi_clear_wake_tx_thd(){
	WAKE_SPI |= (1 << 0);
}

void spi_clear_wake_tx_empty(){
	WAKE_SPI |= (1 << 1);
}

void spi_clear_wake_rx_thd(){
	WAKE_SPI |= (1 << 2);
}

void spi_clear_wake_rx_full(){
	WAKE_SPI |= (1 << 3);
}

void spi_tx_thd_enable_wake(){
	WAKEE_SPI |= (1 << 0);
}

void spi_tx_thd_disable_wake(){
	WAKEE_SPI &= ~(1 << 0);
}

void spi_tx_empty_enable_wake(){
	WAKEE_SPI |= (1 << 1);
}

void spi_tx_empty_disable_wake(){
	WAKEE_SPI &= ~(1 << 1);
}

void spi_rx_thd_enable_wake(){
	WAKEE_SPI |= (1 << 2);
}

void spi_rx_thd_disable_wake(){
	WAKEE_SPI &= ~(1 << 2);
}

void spi_rx_full_enable_wake(){
	WAKEE_SPI |= (1 << 3);
}
void spi_rx_full_disable_wake(){
	WAKEE_SPI &= ~(1 << 3);
}

void spi_enable_tx_fifo(){
	DMA_SPI |= (1 << 6);
}

void spi_disable_tx_fifo(){
	DMA_SPI  &= ~(1 << 6);
}

void spi_enable_rx_fifo(){
	DMA_SPI |= (1 << 22);
}
void spi_disable_rx_fifo(){
	DMA_SPI  &= ~(1 << 22);
}

void spi_set_tx_fifo_level(BYTE level){
	WORD mask = DMA_SPI & ~0x0000001f;
	DMA_SPI = level | mask;
}

void spi_set_rx_fifo_thd(BYTE thd){
	WORD mask = DMA_SPI & ~0x001f0000;
	DMA_SPI = thd | mask;
}

void spi_clear_tx_fifo(){
	DMA_SPI |= (1 << 7);
}

void spi_clear_rx_fifo(){
	DMA_SPI |= (1 << 23);
}

BYTE spi_tx_fifo_cnt(){
	return ((DMA_SPI & ( 0x3f << 8 )) >> 8);
}

void spi_tx_dma_enable(){
	DMA_SPI |= (1 << 15);
}

void spi_tx_dma_disable(){
	DMA_SPI  &= ~(1 << 15);
}

void spi_psize(BYTE size){
	switch(size)
	{
		case 0 : PSIZE_SPI = 0x00;
		         break;
		case 1 : PSIZE_SPI = 0x01;
		         break;
		case 2 : PSIZE_SPI = 0x02;
		         break;
		default : PSIZE_SPI = 0x00;
		          break;
	}
}

void spi_send_byte(BYTE data, BYTE ss){
	switch(ss)
	{
		case 0 : spi_ss_sel(0);
				 spi_push_byte(data);
		       	 spi_start_transaction();
		         while(spi_status()){}
		         spi_ss_desel(0);
		       	 break;
		case 1 : spi_ss_sel(1);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(1);
				 break;
		case 2 : spi_ss_sel(2);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(2);
				 break;
		case 3 : spi_ss_sel(3);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(3);
				 break;
		case 4 : spi_ss_sel(4);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(4);
				 break;
		case 5 : spi_ss_sel(5);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(5);
				 break;
		case 6 : spi_ss_sel(6);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(6);
				 break;
		case 7 : spi_ss_sel(7);
				 spi_push_byte(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(7);
				 break;
		default : spi_ss_sel(1);
			   	  spi_push_byte(data);
				  spi_start_transaction();
				  while(spi_status()){}
				  spi_ss_desel(1);
			      break;
	}
}

void spi_send_half_word(HALF_WORD data, BYTE ss){
	switch(ss)
	{
		case 0 : spi_ss_sel(0);
				 spi_push_half_word(data);
		       	 spi_start_transaction();
		       	 while(spi_status()){}
		       	spi_ss_desel(0);
		       	 break;
		case 1 : spi_ss_sel(1);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(1);
				 break;
		case 2 : spi_ss_sel(2);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(2);
				 break;
		case 3 : spi_ss_sel(3);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(3);
				 break;
		case 4 : spi_ss_sel(4);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(4);
				 break;
		case 5 : spi_ss_sel(5);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(5);
				 break;
		case 6 : spi_ss_sel(6);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(6);
				 break;
		case 7 : spi_ss_sel(7);
				 spi_push_half_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(7);
				 break;
		default : spi_ss_sel(1);
				  spi_push_half_word(data);
			   	  spi_start_transaction();
			   	  while(spi_status()){}
			   	  spi_ss_desel(1);
			      break;
	}
}

void spi_send_word(WORD data, BYTE ss){
	switch(ss)
	{
		case 0 : spi_ss_sel(0);
				 spi_push_word(data);
		       	 spi_start_transaction();
		       	 while(spi_status()){}
		       	spi_ss_desel(0);
		       	 break;
		case 1 : spi_ss_sel(1);
		         spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(1);
				 break;
		case 2 : spi_ss_sel(2);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(2);
				 break;
		case 3 : spi_ss_sel(3);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(3);
				 break;
		case 4 : spi_ss_sel(4);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(4);
				 break;
		case 5 : spi_ss_sel(5);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(5);
				 break;
		case 6 : spi_ss_sel(6);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(6);
				 break;
		case 7 : spi_ss_sel(7);
				 spi_push_word(data);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(7);
				 break;
		default : spi_ss_sel(1);
				  spi_push_word(data);
			   	  spi_start_transaction();
			   	  while(spi_status()){}
			   	  spi_ss_desel(1);
			      break;
	}
}

void spi_setup(){
	spi_enable();
	spi_enable_tx_fifo();
	spi_master_mode();
	spi_ss0_output();
	spi_mode(0);
	spi_four_wire();
	spi_ss_delay1(1);
	spi_ss_delay2(1);
	spi_ss_inactive_delay(1);
	spi_byte_setup();
}

void spi_byte_setup(){
	spi_psize(0);
	spi_char_num_bits(8);
	spi_tx_num_char(1);
}

void spi_half_word_setup(){
	spi_psize(1);
	spi_char_num_bits(8);
	spi_tx_num_char(2);
}

void spi_word_setup(){
	spi_psize(2);
	spi_char_num_bits(8);
	spi_tx_num_char(4);
}
