/*
 * spi.h
 *
 *  Created on: 15 lug 2020
 *      Author: Federico.Pozzana
 */

#ifndef SPI_H_
#define SPI_H_

#include "pulpino.h"

#define SPI_DATA                      ( SPI_BASE_ADDR + 0x0000 )
#define SPI_CTRL1                     ( SPI_BASE_ADDR + 0x0004 )
#define SPI_CTRL2                     ( SPI_BASE_ADDR + 0x0008 )
#define SPI_CTRL3                     ( SPI_BASE_ADDR + 0x000C )
#define SPI_CTRL4                     ( SPI_BASE_ADDR + 0x0010 )
#define SPI_BRG_CTRL                  ( SPI_BASE_ADDR + 0x0014 )
#define SPI_I2S_CTRL                  ( SPI_BASE_ADDR + 0x0018 )
#define SPI_DMA                       ( SPI_BASE_ADDR + 0x001C )
#define SPI_IRQ                       ( SPI_BASE_ADDR + 0x0020 )
#define SPI_IRQE                      ( SPI_BASE_ADDR + 0x0024 )
#define SPI_WAKE                      ( SPI_BASE_ADDR + 0x0028 )
#define SPI_WAKEE                     ( SPI_BASE_ADDR + 0x002C )
#define SPI_STAT                      ( SPI_BASE_ADDR + 0x0030 )
#define SPI_XMEM_CTRL                 ( SPI_BASE_ADDR + 0x0034 )
#define SPI_PSIZE                     ( SPI_BASE_ADDR + 0x0040 )

#define DATA_SPI                      REG(SPI_DATA)
#define CTRL1_SPI                     REG(SPI_CTRL1)
#define CTRL2_SPI                     REG(SPI_CTRL2)
#define CTRL3_SPI                     REG(SPI_CTRL3)
#define CTRL4_SPI                     REG(SPI_CTRL4)
#define BRG_CTRL_SPI                  REG(SPI_BRG_CTRL)
#define I2S_SPI                       REG(SPI_I2S_CTRL)
#define DMA_SPI                       REG(SPI_DMA)
#define IRQ_SPI                       REG(SPI_IRQ)
#define IRQE_SPI                      REG(SPI_IRQE)
#define WAKE_SPI                      REG(SPI_WAKE)
#define WAKEE_SPI                     REG(SPI_WAKEE)
#define STAT_SPI                      REG(SPI_STAT)
#define XMEM_CTRL_SPI                 REG(SPI_XMEM_CTRL)
#define PSIZE_SPI                     REG(SPI_PSIZE)

void spi_push_byte(BYTE data);
void spi_push_half_word(HALF_WORD data);
void spi_push_word(WORD data);

BYTE spi_pull_byte();
HALF_WORD spi_pull_half_word();
WORD spi_pull_word();

void spi_send_byte(BYTE data, BYTE ss);
void spi_send_half_word(HALF_WORD data, BYTE ss);
void spi_send_word(WORD data, BYTE ss);

void spi_setup();

void spi_enable();
void spi_disable();

void spi_master_mode();
void spi_slave_mode();

void spi_ss0_output();
void spi_ss0_input();

void spi_start_transaction();

void spi_ss_ctrl(BYTE ctrl);

void spi_ss_sel(BYTE ss_num);
void spi_ss_desel(BYTE ss_num);

void spi_tx_num_char(HALF_WORD num);
void spi_rx_num_char(HALF_WORD num);

void spi_cpol_zero();
void spi_cpol_one();
void spi_cpha_zero();
void spi_cpha_one();

void spi_mode(BYTE mode);

void spi_char_num_bits(BYTE num);

void spi_three_wire();

void spi_four_wire();

void spi_ss_pol_low(BYTE ss_num);
void spi_ss_pol_high(BYTE ss_num);

void spi_ss_delay1(BYTE num);
void spi_ss_delay2(BYTE num);
void spi_ss_inactive_delay(BYTE num);

void spi_duty_low_ctrl(BYTE ctrl);
void spi_duty_high_ctrl(BYTE ctrl);

void spi_duty_scale(BYTE factor);

void spi_clear_irq_tx_thd();
void spi_clear_irq_tx_empty();
void spi_clear_irq_rx_thd();
void spi_clear_irq_rx_full();

void spi_clear_irq_ss_asserted();
void spi_clear_irq_ss_deasserted();

void spi_clear_irq_multi_master_fail();

void spi_clear_irq_slave_abort();

void spi_clear_irq_master_done();

void spi_clear_irq_tx_overrun();
void spi_clear_irq_tx_underrun();
void spi_clear_irq_rx_overrun();
void spi_clear_irq_rx_underrun();

void spi_tx_thd_enable_irq();
void spi_tx_thd_disable_irq();

void spi_tx_empty_enable_irq();
void spi_tx_empty_disable_irq();

void spi_rx_thd_enable_irq();
void spi_rx_thd_disable_irq();

void spi_rx_full_enable_irq();
void spi_rx_full_disable_irq();

void spi_ss_asserted_enable_irq();
void spi_ss_asserted_disable_irq();

void spi_ss_deasserted_enable_irq();
void spi_ss_deasserted_disable_irq();

void spi_multi_master_fault_enable_irq();
void spi_multi_master_fault_disable_irq();

void spi_slave_abort_enable_irq();
void spi_slave_abort_disable_irq();

void spi_master_done_enable_irq();
void spi_master_done_disable_irq();

void spi_tx_overrun_enable_irq();
void spi_tx_overrun_disable_irq();

void spi_tx_underrun_enable_irq();
void spi_tx_underrun_disable_irq();

void spi_rx_overrun_enable_irq();
void spi_rx_overrun_disable_irq();

void spi_rx_underrun_enable_irq();
void spi_rx_underrun_disable_irq();

BYTE spi_status();

void spi_clear_wake_tx_thd();
void spi_clear_wake_tx_empty();
void spi_clear_wake_rx_thd();
void spi_clear_wake_rx_full();

void spi_tx_thd_enable_wake();
void spi_tx_thd_disable_wake();

void spi_tx_empty_enable_wake();
void spi_tx_empty_disable_wake();

void spi_rx_thd_enable_wake();
void spi_rx_thd_disable_wake();

void spi_rx_full_enable_wake();
void spi_rx_full_disable_wake();

void spi_set_tx_fifo_level(BYTE level);
void spi_set_rx_fifo_thd(BYTE thd);

void spi_enable_tx_fifo();
void spi_disable_tx_fifo();

void spi_enable_rx_fifo();
void spi_disable_rx_fifo();

void spi_clear_tx_fifo();

void spi_clear_rx_fifo();

BYTE spi_tx_fifo_cnt();

void spi_tx_dma_enable();
void spi_tx_dma_disable();

void spi_psize(BYTE size);

void spi_byte_setup();
void spi_half_word_setup();
void spi_word_setup();

#endif /* SPI_H_ */
