/*
 * lpuart.h
 *
 *  Created on: 3 lug 2020
 *      Author: Federico.Pozzana
 */

#ifndef UART_H_
#define UART_H_

#include "pulpino.h"

#define UART_CTRL                     ( UART_BASE_ADDR + 0x0000 )
#define UART_STATUS                   ( UART_BASE_ADDR + 0x0004 )
#define UART_INTEN                    ( UART_BASE_ADDR + 0x0008 )
#define UART_INTFL                    ( UART_BASE_ADDR + 0x000C )
#define UART_CKDIV                    ( UART_BASE_ADDR + 0x0010 )
#define UART_OSR                      ( UART_BASE_ADDR + 0x0014 )
#define UART_TXFIFO                   ( UART_BASE_ADDR + 0x0018 )
#define UART_PNR                      ( UART_BASE_ADDR + 0x001C )
#define UART_DATA                     ( UART_BASE_ADDR + 0x0020 )
#define UART_RESERVED_REG_0           ( UART_BASE_ADDR + 0x0024 )
#define UART_RESERVED_REG_1           ( UART_BASE_ADDR + 0x0028 )
#define UART_RESERVED_REG_2           ( UART_BASE_ADDR + 0x002C )
#define UART_DMA                      ( UART_BASE_ADDR + 0x0030 )
#define UART_WKEN                     ( UART_BASE_ADDR + 0x0034 )
#define UART_WKFL                     ( UART_BASE_ADDR + 0x0038 )

#define CTRL_UART                     REG(UART_CTRL)
#define STATUS_UART                   REG(UART_STATUS)
#define INTEN_UART                    REG(UART_INTEN)
#define INTFL_UART                    REG(UART_INTFL)
#define CKDIV_UART                    REG(UART_CKDIV)
#define OSR_UART                      REG(UART_OSR)
#define TXFIFO_UART                   REG(UART_TXFIFO)
#define PNR_UART                      REG(UART_PNR)
#define DATA_UART                     REG(UART_DATA)
#define RR0_UART                      REG(UART_RESERVED_REG_0)
#define RR1_UART                      REG(UART_RESERVED_REG_1)
#define RR2_UART                      REG(UART_RESERVED_REG_2)
#define DMA_UART                      REG(UART_DMA)
#define WKEN_UART                     REG(UART_WKEN)
#define WKFL_UART                     REG(UART_WKFL)


void uart_parity_enable();
void uart_parity_disable();

void uart_even_parity();
void uart_odd_parity();

void uart_zero_parity();
void uart_one_parity();

void uart_parity_config(BYTE enable, BYTE type, BYTE value);

void uart_char_size(WORD size);

void uart_stop_bit(BYTE stop);

void uart_baud_clk_enable(BYTE enable);

BYTE uart_baud_clk_ready();

void set_baudrate_divisor(WORD divisor);

void osr_zero_set();
void osr_one_set();
void osr_two_set();
void osr_zero_clear();
void osr_one_clear();
void osr_two_clear();

void send_uart(BYTE data);

BYTE recieve_uart();

void enable_hardware_control();
void disable_hardware_control();

BYTE rx_fifo_empty();
BYTE rx_fifo_full();

BYTE tx_fifo_empty();
BYTE tx_fifo_full();

BYTE tx_fifo_busy();
BYTE rx_fifo_busy();

void enable_int_rx_thd();
void disable_int_rx_thd();
void set_rx_fifo_thd(BYTE threshold);
void clear_rx_fifo_thd_int();
BYTE is_rx_fifo_thd();

void enable_int_rx_frame_err();
void disable_int_rx_frame_err();
void clear_rx_frame_err_int();

void enable_int_rx_parity_err();
void disable_int_rx_parity_err();
void clear_rx_parity_err_int();

void enable_int_cts_change();
void disable_int_cts_change();
void clear_cts_change_int();

void enable_int_overrun();
void disable_int_overrun();
void clear_overrun_int();

void enable_int_tx_fifo_one_left();
void disable_int_tx_fifo_one_left();
void clear_tx_fifo_one_left_int();

void enable_int_tx_fifo_half_empty();
void disable_int_tx_fifo_half_empty();
void clear_tx_fifo_half_empty_int();

void cts_sampling_enable();
void cts_sampling_disable();

void tx_flush();
void rx_flush();

void rts_neg_cond(BYTE cond);

void clk_sel(BYTE clock);

void auto_gating_enable();
void auto_gating_diable();

void fractional_div_mode_enable();
void fractional_div_mode_disable();

void rx_dual_edge_enable();
void rx_dual_edge_disable();

void rx_tweak_enable();
void rx_tweak_disable();

void rx_tweak_value(BYTE tweak_value);

BYTE rx_fifo_bytes();
BYTE tx_fifo_bytes();

BYTE tx_fifo_next_data();

BYTE get_cts();
void set_rst(BYTE value);

BYTE parity_err_flag();

void wkup_enable_rx_fifo_not_empty();
void wkup_disable_rx_fifo_not_empty();
BYTE wkup_flag_rx_fifo_not_empty();

void wkup_enable_rx_fifo_full();
void wkup_disable_rx_fifo_full();
BYTE wkup_flag_rx_fifo_full();

void wkup_enable_rx_fifo_thd();
void wkup_disable_rx_fifo_thd();
BYTE wkup_flag_rx_fifo_thd();

void uart_printf(char *array);
void uart_scanf();

#endif /* UART_H_ */
