/*
 * interrupt_handlers.c
 *
 *  Created on: 28 mag 2020
 *      Author: Federico.Pozzana
 */

#include "interrupt_handlers.h"
#include "riscv_1_gp_01.h"
#include "core_rv32.h"

#include "i2c.h"

/////////////////////lfsr start//////////////////////////////////////////////
/////////////////////master write////////////////////////////////////////////

//static BYTE lfsr = 0xAC;
//static BYTE mask = 0x5C;

static HALF_WORD lfsr = 0xACE1;
static HALF_WORD mask = 0xB400;
//static int period = 0;

/////////////////////////////////////////////////////////////////////////////

BYTE lfsr_8(BYTE lfsr, BYTE mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

HALF_WORD lfsr_16(HALF_WORD lfsr, HALF_WORD mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

/////////////////////////////////////////////////////////////////////////////

void enable_interrupt(void) {
	IER = 0xFFFFFFFF;
}

void disable_interrupt(void) {
	IER = 0x00000000;
}

void enable_event(void) {
	EER = 0xFFFFFFFF;
}

void disable_event(void) {
	EER = 0x00000000;
}

//ICP = ICP | something => clear interrupt, not global (interrupt clear pending)

void  __attribute__((interrupt("machine")))
PowerFail_Handler(void) {
   //Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000001;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
BusError_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000002;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_02_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000004;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_03_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000008;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_04_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000010;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C slave firmware//////////////////////////////////////
/////////////////////master write////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
	BYTE test0;
	BYTE test1;
	BYTE test2;
	BYTE test3;
	BYTE test4;
	BYTE test5;
	BYTE test6;

	BYTE operation;

	i2c_clear_irq_stop();
	//test0 = i2c_read_data();
	//test1 = i2c_read_data();
	//test2 = i2c_read_data();
	//test3 = i2c_read_data();
	//test4 = i2c_read_data();
	//test5 = i2c_read_data();
	//test6 = i2c_read_data();

	//lfsr = lfsr_8(lfsr, mask);
	lfsr = lfsr_16(lfsr, mask);
	//takes last three bits to get the operation
	operation = lfsr & 0x0007;

	switch(operation)
	{
	case 1 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			 }
			 break;
	case 2 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			 }
			 break;
	case 3 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1) && test3 == ((lfsr & 0x03fc) >> 2)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			}
			break;
	case 4 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1) && test3 == ((lfsr & 0x03fc) >> 2) && test4 == ((lfsr & 0x07f8) >> 3)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			}
			break;
	case 5 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 test5 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1) && test3 == ((lfsr & 0x03fc) >> 2) && test4 == ((lfsr & 0x07f8) >> 3) && test5 == ((lfsr & 0x0ff0) >> 4)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			}
			break;
	case 6 : test0 = i2c_read_data();
			 test1 = i2c_read_data();
			 test2 = i2c_read_data();
			 test3 = i2c_read_data();
			 test4 = i2c_read_data();
			 test5 = i2c_read_data();
			 test6 = i2c_read_data();
			 if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1) && test3 == ((lfsr & 0x03fc) >> 2) && test4 == ((lfsr & 0x07f8) >> 3) && test5 == ((lfsr & 0x0ff0) >> 4) && test6 == ((lfsr & 0x1fe0) >> 5)){
				 test_check ^= 0x01;
				 test_check ^= 0x01;
			 } else {
				 test_check ^= 0x02;
			}
			break;
	default : test0 = i2c_read_data();
			  test1 = i2c_read_data();
			  test2 = i2c_read_data();
			  test3 = i2c_read_data();
			  test4 = i2c_read_data();
			  test5 = i2c_read_data();
			  test6 = i2c_read_data();
			  if(test0 == 0x01 && test1 == (lfsr & 0x00ff) && test2 == ((lfsr & 0x01fe) >> 1) && test3 == ((lfsr & 0x03fc) >> 2) && test4 == ((lfsr & 0x07f8) >> 3) && test5 == ((lfsr & 0x0ff0) >> 4) && test6 == ((lfsr & 0x1fe0) >> 5)){
			 	 test_check ^= 0x01;
			 	 test_check ^= 0x01;
			  } else {
			 	 test_check ^= 0x02;
			 }
			 break;
	}

	//if(test0 == 0x01 && test1 == lfsr && test2 == lfsr && test3 == lfsr && test4 == lfsr && test5 == lfsr && test6 == lfsr){
	//	test_check ^= 0x01;
	//	test_check ^= 0x01;
	//} else {
	//	test_check ^= 0x02;
	//}

*/
/////////////////////master read/////////////////////////////////////////////
/*
	i2c_clear_irq_slave_read_address_match();

	i2c_write_data(lfsr);
	lfsr = lfsr_8(lfsr,mask);
*/
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C master firmware/////////////////////////////////////
/////////////////////master read/////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
	BYTE test;

	i2c_clear_irq_stop();

	test = i2c_read_data();

	if(test != lfsr){
		test_check ^= 0x02;
		test_check ^= 0x02;
	}

	if(lfsr == 0x45){
		test_check ^= 0x01;
		test_check ^= 0x01;
	} else {
		i2c_master_read(0x01, 0x01);
	}

	lfsr = lfsr_8(lfsr,mask);
*/
/////////////////////////////////////////////////////////////////////////////

	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_05_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000020;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C master firmware/////////////////////////////////////
/////////////////////master write////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

	BYTE operation;
	lfsr = lfsr_16(lfsr, mask);
	//period++;
	//if(lfsr != 0x45){
	//	lfsr = lfsr_8(lfsr, mask);
	if(lfsr != 0xACE1){
		operation = lfsr & 0x0007;
		//i2c_master_write_byte(0x01, 0x01, lfsr);
		//i2c_master_write_six_bytes(0x01, 0x01, lfsr, lfsr, lfsr, lfsr, lfsr, lfsr);
		switch(operation)
		{
		case 1 : i2c_master_write_byte(0x01, 0x01, (lfsr & 0x00ff));
				 break;
		case 2 : i2c_master_write_half_word(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1));
				 break;
		case 3 : i2c_master_write_three_bytes(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1), ((lfsr & 0x03fc) >> 2));
				 break;
		case 4 : i2c_master_write_word(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1), ((lfsr & 0x03fc) >> 2), ((lfsr & 0x07f8) >> 3));
				 break;
		case 5 : i2c_master_write_five_bytes(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1), ((lfsr & 0x03fc) >> 2), ((lfsr & 0x07f8) >> 3), ((lfsr & 0x0ff0) >> 4));
				 break;
		case 6 : i2c_master_write_six_bytes(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1), ((lfsr & 0x03fc) >> 2), ((lfsr & 0x07f8) >> 3), ((lfsr & 0x0ff0) >> 4), ((lfsr & 0x1fe0) >> 5));
				 break;
		default : i2c_master_write_six_bytes(0x01, 0x01, (lfsr & 0x00ff), ((lfsr & 0x01fe) >> 1), ((lfsr & 0x03fc) >> 2), ((lfsr & 0x07f8) >> 3), ((lfsr & 0x0ff0) >> 4), ((lfsr & 0x1fe0) >> 5));
				  break;
		}
	} else {
		test_check = 0x02;
	}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C slave firmware//////////////////////////////////////
/////////////////////master read/////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
	test_check = 0x02;
*/
/////////////////////////////////////////////////////////////////////////////

	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_06_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000040;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C master firmware/////////////////////////////////////
/////////////////////master write////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

	test_check = 0x00;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////I2C slave firmware//////////////////////////////////////
/////////////////////master read/////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
	test_check = 0x00;
*/
/////////////////////////////////////////////////////////////////////////////

	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_07_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000080;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_08_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000100;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_09_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000200;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_10_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000400;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_11_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000800;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_12_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00001000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_13_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00002000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_14_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00004000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_15_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00008000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_16_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00010000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_17_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00020000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_18_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00040000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_19_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00080000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_20_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00100000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_21_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00200000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_22_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00400000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_23_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00800000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_24_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x01000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_25_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x02000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_26_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x04000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_27_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x08000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_28_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x10000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_29_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x20000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_30_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x40000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_31_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x80000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}
