/*
 * interrupt_handlers.c
 *
 *  Created on: 28 mag 2020
 *      Author: Federico.Pozzana
 */

#include "interrupt_handlers.h"
#include "riscv_1_gp_01.h"
#include "core_rv32.h"

#include "spi.h"

/////////////////////lfsr start//////////////////////////////////////////////
/*
//static BYTE lfsr = 0xAC;
//static BYTE mask = 0x5C;
*/
static HALF_WORD lfsr = 0xACE1;
static HALF_WORD mask = 0xB400;

/////////////////////////////////////////////////////////////////////////////

BYTE lfsr_8(BYTE lfsr, BYTE mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

HALF_WORD lfsr_16(HALF_WORD lfsr, HALF_WORD mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

/////////////////////////////////////////////////////////////////////////////

char instruction[INSTRUCTION_LENGHT];
BYTE instruction_ready;

void enable_interrupt(void) {
	IER = 0xFFFFFFFF;
}

void disable_interrupt(void) {
	IER = 0x00000000;
}

void enable_event(void) {
	EER = 0xFFFFFFFF;
}

void disable_event(void) {
	EER = 0x00000000;
}

//ICP = ICP | something => clear interrupt, not global (interrupt clear pending)

void  __attribute__((interrupt("machine")))
PowerFail_Handler(void) {
   //Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000001;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
BusError_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000002;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_02_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000004;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_03_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000008;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_04_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000010;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_05_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000020;
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////SPI master//////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

	int num_tx;

	lfsr = lfsr_16(lfsr, mask);

	if(lfsr == 0xACE1){
		test_check = 0x02;
	}

	WORD lfsr2, lfsr3;

	lfsr2 = lfsr;
	lfsr3 = lfsr;

	lfsr2 = (lfsr2 << 16) | lfsr3;

	//num_tx = (lfsr & 0x000f);
	num_tx = (lfsr & 0x0007);

	switch(num_tx)
		{
		case 0 : spi_tx_num_char(4);
				 spi_ss_sel(1);
				 spi_push_word(lfsr2);
				 spi_start_transaction();
				 while(spi_status()){}
				 spi_ss_desel(1);
				 break;
		case 1 : spi_tx_num_char(8);
		 	 	 spi_ss_sel(1);
		 	 	 spi_push_half_word(lfsr2);
		 	 	 spi_push_half_word((lfsr2 >> 1));
		 	 	 spi_start_transaction();
		 	 	 while(spi_status()){}
		 	 	 spi_ss_desel(1);
		 	 	 break;
		case 2 : spi_tx_num_char(12);
	 	 	 	 spi_ss_sel(1);
	 	 	 	 spi_push_half_word(lfsr2);
	 	 	 	 spi_push_half_word((lfsr2 >> 1));
	 	 	 	 spi_push_half_word((lfsr2 >> 2));
	 	 	 	 spi_start_transaction();
	 	 	 	 while(spi_status()){}
	 	 	 	 spi_ss_desel(1);
	 	 	 	 break;
		case 3 : spi_tx_num_char(16);
		 	 	 spi_ss_sel(1);
		 	 	 spi_push_half_word(lfsr2);
		 	 	 spi_push_half_word((lfsr2 >> 1));
		 	 	 spi_push_half_word((lfsr2 >> 2));
		 	 	 spi_push_half_word((lfsr2 >> 3));
		 	 	 spi_start_transaction();
		 	 	 while(spi_status()){}
		 	 	 spi_ss_desel(1);
		 	 	 break;
		case 4 : spi_tx_num_char(20);
	 	 	 	 spi_ss_sel(1);
	 	 	 	 spi_push_half_word(lfsr2);
	 	 	 	 spi_push_half_word((lfsr2 >> 1));
	 	 	 	 spi_push_half_word((lfsr2 >> 2));
	 	 	 	 spi_push_half_word((lfsr2 >> 3));
	 	 	 	 spi_push_half_word((lfsr2 >> 4));
	 	 	 	 spi_start_transaction();
	 	 	 	 while(spi_status()){}
	 	 	 	 spi_ss_desel(1);
	 	 	 	 break;
		case 5 : spi_tx_num_char(24);
		 	 	 spi_ss_sel(1);
		 	 	 spi_push_half_word(lfsr2);
		 	 	 spi_push_half_word((lfsr2 >> 1));
		 	 	 spi_push_half_word((lfsr2 >> 2));
		 	 	 spi_push_half_word((lfsr2 >> 3));
		 	 	 spi_push_half_word((lfsr2 >> 4));
		 	 	 spi_push_half_word((lfsr2 >> 5));
		 	 	 spi_start_transaction();
		 	 	 while(spi_status()){}
		 	 	 spi_ss_desel(1);
		 	 	 break;
		case 6 : spi_tx_num_char(28);
	 	 	 	 spi_ss_sel(1);
	 	 	 	 spi_push_half_word(lfsr2);
	 	 	 	 spi_push_half_word((lfsr2 >> 1));
	 	 	 	 spi_push_half_word((lfsr2 >> 2));
	 	 	 	 spi_push_half_word((lfsr2 >> 3));
	 	 	 	 spi_push_half_word((lfsr2 >> 4));
	 	 	 	 spi_push_half_word((lfsr2 >> 5));
	 	 	 	 spi_push_half_word((lfsr2 >> 6));
	 	 	 	 spi_start_transaction();
	 	 	 	 while(spi_status()){}
	 	 	 	 spi_ss_desel(1);
	 	 	 	 break;
		case 7 : spi_tx_num_char(32);
		 	 	 spi_ss_sel(1);
		 	 	 spi_push_half_word(lfsr2);
		 	 	 spi_push_half_word((lfsr2 >> 1));
		 	 	 spi_push_half_word((lfsr2 >> 2));
		 	 	 spi_push_half_word((lfsr2 >> 3));
		 	 	 spi_push_half_word((lfsr2 >> 4));
		 	 	 spi_push_half_word((lfsr2 >> 5));
		 	 	 spi_push_half_word((lfsr2 >> 6));
		 	 	 spi_push_half_word((lfsr2 >> 7));
		 	 	 spi_start_transaction();
		 	 	 while(spi_status()){}
		 	 	 spi_ss_desel(1);
		 	 	 break;
		 /*
		case 8 : spi_tx_num_char(18);
	 	 	 	 spi_ss_sel(1);
	 	 	 	 spi_push_half_word(lfsr);
	 	 	 	 spi_push_half_word((lfsr >> 1));
	 	 	 	 spi_push_half_word((lfsr >> 2));
	 	 	 	 spi_push_half_word((lfsr >> 3));
	 	 	 	 spi_push_half_word((lfsr >> 4));
	 	 	 	 spi_push_half_word((lfsr >> 5));
	 	 	 	 spi_push_half_word((lfsr >> 6));
	 	 	 	 spi_push_half_word((lfsr >> 7));
	 	 	 	 spi_push_half_word((lfsr >> 8));
	 	 	 	 spi_start_transaction();
	 	 	 	 while(spi_status()){}
	 	 	 	 spi_ss_desel(1);
	 	 	 	 break;
		case 9 : spi_tx_num_char(20);
		 	 	 spi_ss_sel(1);
		 	 	 spi_push_half_word(lfsr);
		 	 	 spi_push_half_word((lfsr >> 1));
		 	 	 spi_push_half_word((lfsr >> 2));
		 	 	 spi_push_half_word((lfsr >> 3));
		 	 	 spi_push_half_word((lfsr >> 4));
		 	 	 spi_push_half_word((lfsr >> 5));
		 	 	 spi_push_half_word((lfsr >> 6));
		 	 	 spi_push_half_word((lfsr >> 7));
		 	 	 spi_push_half_word((lfsr >> 8));
		 	 	 spi_push_half_word((lfsr >> 9));
		 	 	 spi_start_transaction();
		 	 	 while(spi_status()){}
		 	 	 spi_ss_desel(1);
		 	 	 break;
		case 10 : spi_tx_num_char(22);
	 	 	 	  spi_ss_sel(1);
	 	 	 	  spi_push_half_word(lfsr);
	 	 	 	  spi_push_half_word((lfsr >> 1));
	 	 	 	  spi_push_half_word((lfsr >> 2));
	 	 	 	  spi_push_half_word((lfsr >> 3));
	 	 	 	  spi_push_half_word((lfsr >> 4));
	 	 	 	  spi_push_half_word((lfsr >> 5));
	 	 	 	  spi_push_half_word((lfsr >> 6));
	 	 	 	  spi_push_half_word((lfsr >> 7));
	 	 	 	  spi_push_half_word((lfsr >> 8));
	 	 	 	  spi_push_half_word((lfsr >> 9));
	 	 	 	  spi_push_half_word((lfsr >> 10));
	 	 	 	  spi_start_transaction();
	 	 	 	  while(spi_status()){}
	 	 	 	  spi_ss_desel(1);
	 	 	 	  break;
		case 11 : spi_tx_num_char(24);
	 	  	  	  spi_ss_sel(1);
	 	  	  	  spi_push_half_word(lfsr);
	 	  	  	  spi_push_half_word((lfsr >> 1));
	 	  	  	  spi_push_half_word((lfsr >> 2));
	 	  	  	  spi_push_half_word((lfsr >> 3));
	 	  	  	  spi_push_half_word((lfsr >> 4));
	 	  	  	  spi_push_half_word((lfsr >> 5));
	 	  	  	  spi_push_half_word((lfsr >> 6));
	 	  	  	  spi_push_half_word((lfsr >> 7));
	 	  	  	  spi_push_half_word((lfsr >> 8));
	 	  	  	  spi_push_half_word((lfsr >> 9));
	 	  	  	  spi_push_half_word((lfsr >> 10));
	 	  	  	  spi_push_half_word((lfsr >> 11));
	 	  	  	  spi_start_transaction();
	 	  	  	  while(spi_status()){}
	 	  	  	  spi_ss_desel(1);
	 	  	  	  break;
		case 12 : spi_tx_num_char(26);
		  	  	  spi_ss_sel(1);
		  	  	  spi_push_half_word(lfsr);
		  	  	  spi_push_half_word((lfsr >> 1));
		  	  	  spi_push_half_word((lfsr >> 2));
		  	  	  spi_push_half_word((lfsr >> 3));
		  	  	  spi_push_half_word((lfsr >> 4));
		  	  	  spi_push_half_word((lfsr >> 5));
		  	  	  spi_push_half_word((lfsr >> 6));
		  	  	  spi_push_half_word((lfsr >> 7));
		  	  	  spi_push_half_word((lfsr >> 8));
		  	  	  spi_push_half_word((lfsr >> 9));
		  	  	  spi_push_half_word((lfsr >> 10));
		  	  	  spi_push_half_word((lfsr >> 11));
		  	  	  spi_push_half_word((lfsr >> 12));
		  	  	  spi_start_transaction();
		  	  	  while(spi_status()){}
		  	  	  spi_ss_desel(1);
		  	  	  break;
		case 13 : spi_tx_num_char(28);
	  	  	  	  spi_ss_sel(1);
	  	  	  	  spi_push_half_word(lfsr);
	  	  	  	  spi_push_half_word((lfsr >> 1));
	  	  	  	  spi_push_half_word((lfsr >> 2));
	  	  	  	  spi_push_half_word((lfsr >> 3));
	  	  	  	  spi_push_half_word((lfsr >> 4));
	  	  	  	  spi_push_half_word((lfsr >> 5));
	  	  	  	  spi_push_half_word((lfsr >> 6));
	  	  	  	  spi_push_half_word((lfsr >> 7));
	  	  	  	  spi_push_half_word((lfsr >> 8));
	  	  	  	  spi_push_half_word((lfsr >> 9));
	  	  	  	  spi_push_half_word((lfsr >> 10));
	  	  	  	  spi_push_half_word((lfsr >> 11));
	  	  	  	  spi_push_half_word((lfsr >> 12));
	  	  	  	  spi_push_half_word((lfsr >> 13));
	  	  	  	  spi_start_transaction();
	  	  	  	  while(spi_status()){}
	  	  	  	  spi_ss_desel(1);
	  	  	  	  break;
		case 14 : spi_tx_num_char(30);
		  	  	  spi_ss_sel(1);
		  	  	  spi_push_half_word(lfsr);
		  	  	  spi_push_half_word((lfsr >> 1));
		  	  	  spi_push_half_word((lfsr >> 2));
		  	  	  spi_push_half_word((lfsr >> 3));
		  	  	  spi_push_half_word((lfsr >> 4));
		  	  	  spi_push_half_word((lfsr >> 5));
		  	  	  spi_push_half_word((lfsr >> 6));
		  	  	  spi_push_half_word((lfsr >> 7));
		  	  	  spi_push_half_word((lfsr >> 8));
		  	  	  spi_push_half_word((lfsr >> 9));
		  	  	  spi_push_half_word((lfsr >> 10));
		  	  	  spi_push_half_word((lfsr >> 11));
		  	  	  spi_push_half_word((lfsr >> 12));
		  	  	  spi_push_half_word((lfsr >> 13));
		  	  	  spi_push_half_word((lfsr >> 14));
		  	  	  spi_start_transaction();
		  	  	  while(spi_status()){}
		  	  	  spi_ss_desel(1);
		  	  	  break;
		case 15 : spi_tx_num_char(32);
	  	  	  	  spi_ss_sel(1);
	  	  	  	  spi_push_half_word(lfsr);
	  	  	  	  spi_push_half_word((lfsr >> 1));
	  	  	  	  spi_push_half_word((lfsr >> 2));
	  	  	  	  spi_push_half_word((lfsr >> 3));
	  	  	  	  spi_push_half_word((lfsr >> 4));
	  	  	  	  spi_push_half_word((lfsr >> 5));
	  	  	  	  spi_push_half_word((lfsr >> 6));
	  	  	  	  spi_push_half_word((lfsr >> 7));
	  	  	  	  spi_push_half_word((lfsr >> 8));
	  	  	  	  spi_push_half_word((lfsr >> 9));
	  	  	  	  spi_push_half_word((lfsr >> 10));
	  	  	  	  spi_push_half_word((lfsr >> 11));
	  	  	  	  spi_push_half_word((lfsr >> 12));
	  	  	  	  spi_push_half_word((lfsr >> 13));
	  	  	  	  spi_push_half_word((lfsr >> 14));
	  	  	  	  spi_push_half_word((lfsr >> 15));
	  	  	  	  spi_start_transaction();
	  	  	  	  while(spi_status()){}
	  	  	  	  spi_ss_desel(1);
	  	  	  	  break;
	  	  	  */
	}

/*
    num_tx = (lfsr & 0x001f);

	switch(num_tx)
	{
	case 0 : spi_tx_num_char(1);
			 spi_ss_sel(1);
			 spi_push_byte((lfsr & 0x00ff));
			 spi_start_transaction();
			 while(spi_status()){}
			 spi_ss_desel(1);
			 break;
	case 1 : spi_tx_num_char(2);
	 	 	 spi_ss_sel(1);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
	 	 	 spi_start_transaction();
	 	 	 while(spi_status()){}
	 	 	 spi_ss_desel(1);
	 	 	 break;
	case 2 : spi_tx_num_char(3);
 	 	 	 spi_ss_sel(1);
 	 	 	 spi_push_byte((lfsr & 0x00ff));
 	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
 	 	 	 spi_start_transaction();
 	 	 	 while(spi_status()){}
 	 	 	 spi_ss_desel(1);
 	 	 	 break;
	case 3 : spi_tx_num_char(4);
	 	 	 spi_ss_sel(1);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
	 	 	 spi_start_transaction();
	 	 	 while(spi_status()){}
	 	 	 spi_ss_desel(1);
	 	 	 break;
	case 4 : spi_tx_num_char(5);
 	 	 	 spi_ss_sel(1);
 	 	 	 spi_push_byte((lfsr & 0x00ff));
 	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
 	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
 	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
 	 	 	 spi_start_transaction();
 	 	 	 while(spi_status()){}
 	 	 	 spi_ss_desel(1);
 	 	 	 break;
	case 5 : spi_tx_num_char(6);
	 	 	 spi_ss_sel(1);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
	 	 	 spi_push_byte((lfsr & 0x1fe0) >> 5);
	 	 	 spi_start_transaction();
	 	 	 while(spi_status()){}
	 	 	 spi_ss_desel(1);
	 	 	 break;
	case 6 : spi_tx_num_char(7);
 	 	 	 spi_ss_sel(1);
 	 	 	 spi_push_byte((lfsr & 0x00ff));
 	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
 	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
 	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
 	 	 	 spi_push_byte((lfsr & 0x1fe0) >> 5);
 	 	 	 spi_push_byte((lfsr & 0x3fc0) >> 6);
 	 	 	 spi_start_transaction();
 	 	 	 while(spi_status()){}
 	 	 	 spi_ss_desel(1);
 	 	 	 break;
	case 7 : spi_tx_num_char(8);
	 	 	 spi_ss_sel(1);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
	 	 	 spi_push_byte((lfsr & 0x1fe0) >> 5);
	 	 	 spi_push_byte((lfsr & 0x3fc0) >> 6);
	 	 	 spi_push_byte((lfsr & 0x7f80) >> 7);
	 	 	 spi_start_transaction();
	 	 	 while(spi_status()){}
	 	 	 spi_ss_desel(1);
	 	 	 break;
	case 8 : spi_tx_num_char(9);
 	 	 	 spi_ss_sel(1);
 	 	 	 spi_push_byte((lfsr & 0x00ff));
 	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
 	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
 	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
 	 	 	 spi_push_byte((lfsr & 0x1fe0) >> 5);
 	 	 	 spi_push_byte((lfsr & 0x3fc0) >> 6);
 	 	 	 spi_push_byte((lfsr & 0x7f80) >> 7);
 	 	 	 spi_push_byte((lfsr & 0xff00) >> 8);
 	 	 	 spi_start_transaction();
 	 	 	 while(spi_status()){}
 	 	 	 spi_ss_desel(1);
 	 	 	 break;
	case 9 : spi_tx_num_char(10);
	 	 	 spi_ss_sel(1);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_push_byte((lfsr & 0x01fe) >> 1);
	 	 	 spi_push_byte((lfsr & 0x03fc) >> 2);
	 	 	 spi_push_byte((lfsr & 0x07f8) >> 3);
	 	 	 spi_push_byte((lfsr & 0x0ff0) >> 4);
	 	 	 spi_push_byte((lfsr & 0x1fe0) >> 5);
	 	 	 spi_push_byte((lfsr & 0x3fc0) >> 6);
	 	 	 spi_push_byte((lfsr & 0x7f80) >> 7);
	 	 	 spi_push_byte((lfsr & 0xff00) >> 8);
	 	 	 spi_push_byte((lfsr & 0x00ff));
	 	 	 spi_start_transaction();
	 	 	 while(spi_status()){}
	 	 	 spi_ss_desel(1);
	 	 	 break;
	case 10 : spi_tx_num_char(11);
 	 	 	  spi_ss_sel(1);
 	 	 	  spi_push_byte((lfsr & 0x00ff));
 	 	 	  spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	  spi_push_byte((lfsr & 0x03fc) >> 2);
 	 	 	  spi_push_byte((lfsr & 0x07f8) >> 3);
 	 	 	  spi_push_byte((lfsr & 0x0ff0) >> 4);
 	 	 	  spi_push_byte((lfsr & 0x1fe0) >> 5);
 	 	 	  spi_push_byte((lfsr & 0x3fc0) >> 6);
 	 	 	  spi_push_byte((lfsr & 0x7f80) >> 7);
 	 	 	  spi_push_byte((lfsr & 0xff00) >> 8);
 	 	 	  spi_push_byte((lfsr & 0x00ff));
 	 	 	  spi_push_byte((lfsr & 0x01fe) >> 1);
 	 	 	  spi_start_transaction();
 	 	 	  while(spi_status()){}
 	 	 	  spi_ss_desel(1);
 	 	 	  break;
	case 11 : spi_tx_num_char(12);
 	  	  	  spi_ss_sel(1);
 	  	  	  spi_push_byte((lfsr & 0x00ff));
 	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
 	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
 	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
 	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
 	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
 	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
 	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
 	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
 	  	  	  spi_push_byte((lfsr & 0x00ff));
 	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
 	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
 	  	  	  spi_start_transaction();
 	  	  	  while(spi_status()){}
 	  	  	  spi_ss_desel(1);
 	  	  	  break;
	case 12 : spi_tx_num_char(13);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 13 : spi_tx_num_char(14);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	      spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 14 : spi_tx_num_char(15);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 15 : spi_tx_num_char(16);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	      spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 16 : spi_tx_num_char(17);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 17 : spi_tx_num_char(18);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 18 : spi_tx_num_char(19);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 19 : spi_tx_num_char(20);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 20 : spi_tx_num_char(21);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 21 : spi_tx_num_char(22);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 22 : spi_tx_num_char(23);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_start_transaction();
	  	  	  while(spi_status()){}
	  	  	  spi_ss_desel(1);
	  	  	  break;
	case 23 : spi_tx_num_char(24);
  	  	  	  spi_ss_sel(1);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
  	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
  	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
  	  	  	  spi_push_byte((lfsr & 0x00ff));
  	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
  	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
  	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
  	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
  	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
  	  	  	  spi_start_transaction();
  	  	  	  while(spi_status()){}
  	  	  	  spi_ss_desel(1);
  	  	  	  break;
	case 24 : spi_tx_num_char(25);
	  	  	  spi_ss_sel(1);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
	  	  	  spi_push_byte((lfsr & 0x07f8) >> 3);
	  	  	  spi_push_byte((lfsr & 0x0ff0) >> 4);
	  	  	  spi_push_byte((lfsr & 0x1fe0) >> 5);
	  	  	  spi_push_byte((lfsr & 0x3fc0) >> 6);
	  	  	  spi_push_byte((lfsr & 0x7f80) >> 7);
	  	  	  spi_push_byte((lfsr & 0xff00) >> 8);
	  	  	  spi_push_byte((lfsr & 0x00ff));
	  	  	  spi_push_byte((lfsr & 0x01fe) >> 1);
	  	  	  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 25 : spi_tx_num_char(26);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 26 : spi_tx_num_char(27);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 27 : spi_tx_num_char(28);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 28 : spi_tx_num_char(29);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 29 : spi_tx_num_char(30);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 30 : spi_tx_num_char(31);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	case 31 : spi_tx_num_char(32);
			  spi_ss_sel(1);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_push_byte((lfsr & 0x1fe0) >> 5);
			  spi_push_byte((lfsr & 0x3fc0) >> 6);
			  spi_push_byte((lfsr & 0x7f80) >> 7);
			  spi_push_byte((lfsr & 0xff00) >> 8);
			  spi_push_byte((lfsr & 0x00ff));
			  spi_push_byte((lfsr & 0x01fe) >> 1);
			  spi_push_byte((lfsr & 0x03fc) >> 2);
			  spi_push_byte((lfsr & 0x07f8) >> 3);
			  spi_push_byte((lfsr & 0x0ff0) >> 4);
			  spi_start_transaction();
			  while(spi_status()){}
			  spi_ss_desel(1);
			  break;
	}
*/
///////////////////////////////////////////////////////////////////////////////////////////
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_06_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000040;
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////SPI master//////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

	test_check = 0x00;

///////////////////////////////////////////////////////////////////////////////////////////
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_07_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000080;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_08_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000100;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_09_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000200;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_10_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000400;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_11_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000800;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_12_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00001000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_13_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00002000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_14_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00004000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_15_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00008000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_16_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00010000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_17_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00020000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_18_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00040000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_19_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00080000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_20_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00100000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_21_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00200000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_22_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00400000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_23_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00800000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_24_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x01000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_25_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x02000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_26_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x04000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_27_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x08000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_28_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x10000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_29_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x20000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_30_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x40000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_31_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x80000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}
