/*
 * interrupt_handlers.c
 *
 *  Created on: 28 mag 2020
 *      Author: Federico.Pozzana
 */

#include "interrupt_handlers.h"
#include "riscv_1_gp_01.h"
#include "core_rv32.h"

#include "spi.h"

/////////////////////lfsr start//////////////////////////////////////////////

static HALF_WORD lfsr = 0xACE1;
static HALF_WORD mask = 0xB400;

/////////////////////////////////////////////////////////////////////////////

BYTE lfsr_8(BYTE lfsr, BYTE mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

HALF_WORD lfsr_16(HALF_WORD lfsr, HALF_WORD mask){
	int lsb;

	lsb = lfsr & 1;
	lfsr >>= 1;

	if (lsb == 1){
		lfsr ^= mask;
	}

	return lfsr;
}

/////////////////////////////////////////////////////////////////////////////

char instruction[INSTRUCTION_LENGHT];
BYTE instruction_ready;

void enable_interrupt(void) {
	IER = 0xFFFFFFFF;
}

void disable_interrupt(void) {
	IER = 0x00000000;
}

void enable_event(void) {
	EER = 0xFFFFFFFF;
}

void disable_event(void) {
	EER = 0x00000000;
}

//ICP = ICP | something => clear interrupt, not global (interrupt clear pending)

void  __attribute__((interrupt("machine")))
PowerFail_Handler(void) {
   //Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000001;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
BusError_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000002;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_02_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000004;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_03_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000008;
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////SPI slave///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

	//BYTE data;
	//HALF_WORD data;
	WORD data;
	BYTE bytes;

	WORD lfsr2, lfsr3;

	spi_clear_irq_ss_deasserted();
	lfsr = lfsr_16(lfsr, mask);
	bytes = spi_rx_fifo_cnt();

	//lfsr2 = (lfsr << 16) & (lfsr);
	lfsr2 = lfsr;
	lfsr3 = lfsr;

	lfsr2 = (lfsr2 << 16) | lfsr3;


	for(int i = 0; i < bytes/4; i++){
		//data = spi_pull_byte();
		//data = spi_pull_half_word();
		data = spi_pull_word();
		switch(i)
		{
		case 0 : //if(data != (lfsr & 0x00ff)){
				 if(data != lfsr2){
				 	 test_check = 0x02;
				 }
				 break;
		case 1 : //if(data != ((lfsr & 0x01fe) >> 1)){
				 if(data != (lfsr2 >> 1)){
		 	 	 	 test_check = 0x02;
		 	 	 }
		 	 	 break;
		case 2 : //if(data != ((lfsr & 0x03fc) >> 2)){
				 if(data != (lfsr2 >> 2)){
	 	 	 	 	 test_check = 0x02;
	 	 	 	 }
	 	 	 	 break;
		case 3 : //if(data != ((lfsr & 0x07f8) >> 3)){
			 	 if(data != (lfsr2 >> 3)){
 	 	 	 	 	 test_check = 0x02;
 	 	 	 	 }
 	 	 	 	 break;
		case 4 : //if(data != ((lfsr & 0x0ff0) >> 4)){
				 if(data != (lfsr2 >> 4)){
	 	 	 	 	 test_check = 0x02;
	 	 	 	 }
	 	 	 	 break;
		case 5 : //if(data != ((lfsr & 0x1fe0) >> 5)){
				 if(data != (lfsr2 >> 5)){
 	 	 	 	 	 test_check = 0x02;
 	 	 	 	 }
 	 	 	 	 break;
		case 6 : //if(data != ((lfsr & 0x3fc0) >> 6)){
				 if(data != (lfsr2 >> 6)){
	 	 	 	 	 test_check = 0x02;
	 	 	 	 }
	 	 	 	 break;
		case 7 : //if(data != ((lfsr & 0x7f80) >> 7)){
				 if(data != (lfsr2 >> 7)){
 	 	 	 	 	 test_check = 0x02;
 	 	 	 	 }
				 break;
		/*
		case 8 : //if(data != ((lfsr & 0xff00) >> 8)){
				 if(data != (lfsr >> 8)){
	 	 	 	 	 test_check = 0x02;
	 	 	 	 }
		 	 	 break;
		case 9 : //if(data != (lfsr & 0x00ff)){
				 if(data != (lfsr >> 9)){
 	 	 	 	 	 test_check = 0x02;
 	 	 	 	 }
	 	 	 	 break;
		case 10 : //if(data != ((lfsr & 0x01fe) >> 1)){
				  if(data != (lfsr >> 10)){
	 	 	 	  	  test_check = 0x02;
	 	 	  	  }
 	 	  	  	  break;
		case 11 : //if(data != ((lfsr & 0x03fc) >> 2)){
				  if(data != (lfsr >> 11)){
	 	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 12 : //if(data != ((lfsr & 0x07f8) >> 3)){
				  if(data != (lfsr >> 12)){
 	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 13 : //if(data != ((lfsr & 0x0ff0) >> 4)){
				  if(data != (lfsr >> 13)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 14 : //if(data != ((lfsr & 0x1fe0) >> 5)){
				  if(data != (lfsr >> 14)){
  	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 15 : //if(data != ((lfsr & 0x3fc0) >> 6)){
				  if(data != (lfsr >> 15)){
				  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
  	  	  	  	  */
  	  	  	  	  /*
		case 16 : if(data != ((lfsr & 0x7f80) >> 7)){
		  	  	  	  test_check = 0x02;
  	  	  	  	  }
				  break;
		case 17 : if(data != ((lfsr & 0xff00) >> 8)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
		  	  	  break;
		case 18 : if(data != (lfsr & 0x00ff)){
	 	 	 	  	  test_check = 0x02;
	 	 	  	  }
 	 	  	  	  break;
		case 19 : if(data != ((lfsr & 0x01fe) >> 1)){
	 	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 20 : if(data != ((lfsr & 0x03fc) >> 2)){
 	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
				  break;
		case 21 : if(data != ((lfsr & 0x07f8) >> 3)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 22 : if(data != ((lfsr & 0x0ff0) >> 4)){
  	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 23 : if(data != ((lfsr & 0x1fe0) >> 5)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 24 : if(data != ((lfsr & 0x3fc0) >> 6)){
		  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 25 : if(data != ((lfsr & 0x7f80) >> 7)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
		  	  	  break;
		case 26 : if(data != ((lfsr & 0xff00) >> 8)){
  	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 27 : if(data != (lfsr & 0x00ff)){
	 	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		case 28 : if(data != ((lfsr & 0x01fe) >> 1)){
 	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 29 : if(data != ((lfsr & 0x03fc) >> 2)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
		  	  	  break;
		case 30 : if(data != ((lfsr & 0x07f8) >> 3)){
  	  	  	  	  	  test_check = 0x02;
  	  	  	  	  }
	  	  	  	  break;
		case 31 : if(data != ((lfsr & 0x0ff0) >> 4)){
	  	  	  	  	  test_check = 0x02;
	  	  	  	  }
  	  	  	  	  break;
		*/
		}

	}


	spi_clear_rx_fifo();

	test_check ^= 0x01;
	test_check ^= 0x01;

///////////////////////////////////////////////////////////////////////////////////////////
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_04_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000010;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_05_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000020;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_06_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000040;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_07_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000080;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_08_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000100;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_09_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000200;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_10_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000400;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_11_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00000800;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_12_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00001000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_13_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00002000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_14_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00004000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_15_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00008000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_16_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00010000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_17_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00020000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_18_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00040000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_19_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00080000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_20_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00100000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_21_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00200000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_22_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00400000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_23_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x00800000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_24_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x01000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_25_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x02000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_26_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x04000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_27_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x08000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_28_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x10000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_29_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x20000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_30_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x40000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}

void  __attribute__((interrupt("machine")))
IRQ_31_Handler(void) {
	//Disable interrupts, clear flag and re-enable interrupts
	//__disable_irq();
	IER = 0x00000000;
	ICP = ICP | 0x80000000;
	IER = 0xFFFFFFFF;
	//__enable_irq();
}
